package common;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>処理高速化のためにオブジェクトをプールするためのクラス。</p>
 * <p>本格的に使用するのであれば
 * <a href="http://commons.apache.org/pool/">Apache Commons Pool</a>
 * などを導入すべき。</p>
 * @author Honeplus
 * @version $Id: ObjectPool.java 32 2009-03-01 13:59:59Z Honeplus $
 */
public final class ObjectPool {

	/**
	 * {@link ObjectPool} のキャッシュキー値を定義するためのクラス。
	 * @author Honeplus
	 * @version $Id: ObjectPool.java 32 2009-03-01 13:59:59Z Honeplus $
	 */
	private static class CacheKey {

		/** オブジェクトのクラス名。 */
		private Class<?> type;
		/** オブジェクトのコンストラクタへの引数。 */
		private Object[] params;

		/**
		 * {@link ObjectPool} のキャッシュ用キーを作成。
		 * @param clazz オブジェクトのクラス名
		 * @param param オブジェクトのコンストラクタへの引数
		 * @throws NullPointerException <var>clazz</var> が <code>null</code>
		 */
		public CacheKey(final Class<?> clazz, final Object... param) {
			if (clazz == null) {
				throw new NullPointerException("clazz is null");
			}
			this.type = clazz;
			this.params = param;
		}

		/**
		 * このオブジェクトと「等価」になるオブジェクトがあるかどうかを示します。
		 * @param obj 比較対象の参照オブジェクト
		 * @return 引数に指定されたオブジェクトとこのオブジェクトが等しい場合は
		 *         <code>true</code>、そうでない場合は <code>false</code>
		 */
		@Override
		public boolean equals(final Object obj) {
			// nullチェック
			if (obj == null) {
				return false;
			}
			// 型チェック
			if (!(obj instanceof CacheKey)) {
				return false;
			}
			CacheKey other = (CacheKey) obj;
			// クラス名チェック
			if (!type.equals(other.type)) {
				return false;
			}
			// パラメータチェック
			if (!Arrays.equals(params, other.params)) {
				return false;
			}
			return true;
		}

		/**
		 * <p>オブジェクトのハッシュコード値を返します。
		 * このメソッドは、{@link Hashtable} によって提供されるようなハッシュテーブルで使用するために用意されています。</p>
		 * <p>{@link #name}, {@link #params} を文字列にして連結し、その
		 * {@link String#hashCode()} を返す。必ずしも一意ではないので注意。</p>
		 * @return このオブジェクトのハッシュコード値
		 */
		@Override
		public int hashCode() {
			// 文字列にして連結し、そのhashCode()を返す
			StringBuilder b = new StringBuilder(type.toString());
			if (params != null) {
				for (Object o : params) {
					b.append(',');
					if (o != null) {
						b.append(o);
					} else {
						b.append("null");
					}
				}
			}
			return b.toString().hashCode();
		}
	};

	/** オブジェクトを全アプリケーションでキャッシュするためのマップ。 */
	private static Map<CacheKey, Object> caches
		= Collections.synchronizedMap(new HashMap<CacheKey, Object>());

	/**
	 * インスタンス作成抑止用コンストラクタ。
	 * @throws UnsupportedOperationException 実行不可
	 */
	private ObjectPool() {
		throw new UnsupportedOperationException("this class is utility");
	}

	/**
	 * <p>オブジェクトのインスタンスを取得。
	 * キャッシュを保持しており、キャッシュがあればそこから取得。</p>
	 * <p><var>constructor</var> が指定されている場合、
	 * そのコンストラクタでオブジェクトを生成する。
	 * 指定されていない場合、パラメータに合うコンストラクタを探索し使用する。
	 * ただし、複雑なコンストラクタを持つクラスの場合、判別できない可能性がある。</p>
	 * @param clazz オブジェクトのクラス
	 * @param constructor オブジェクトの生成で使用するコンストラクタ
	 * @param param オブジェクトのコンストラクタへの引数
	 * @return オブジェクトのインスタンス
	 * @throws NullPointerException <var>clazz</var> が <code>null</code>
	 * @throws NoSuchMethodException 一致するコンストラクタが見つからない場合
	 * @throws InvocationTargetException コンストラクタで例外がスローされた場合
	 * @throws IllegalAccessException コンストラクタにアクセスできない場合
	 * @throws InstantiationException クラスが <code>abstract</code> クラスを表す場合
	 */
	private static synchronized Object getInstance(
			final Class<?> clazz, final Constructor<?> constructor,
			final Object... param) throws InstantiationException,
			IllegalAccessException, InvocationTargetException,
			NoSuchMethodException {

		// キャッシュに存在するかのチェック
		CacheKey key = new CacheKey(clazz, param);
		Object obj = caches.get(key);
		if (obj != null) {
			// キャッシュにあれば、その値を返す
			return obj;
		}
		// キャッシュにない場合、オブジェクトを生成して返す
		if (constructor == null) {
			// コンストラクタがない場合、探索して作成する
			obj = ObjectUtility.newInstance(clazz, param);
		} else {
			// コンストラクタがある場合、それを使用する
			obj = constructor.newInstance(param);
		}
		// 生成したオブジェクトはキャッシュに登録
		caches.put(key, obj);
		return obj;
	}

	/**
	 * オブジェクトのインスタンスを取得。
	 * キャッシュを保持しており、キャッシュがあればそこから取得。
	 * @param constructor オブジェクトの生成で使用するコンストラクタ
	 * @param param オブジェクトのコンストラクタへの引数
	 * @return オブジェクトのインスタンス
	 * @throws NullPointerException <var>clazz</var> が <code>null</code>
	 * @throws NoSuchMethodException 一致するコンストラクタが見つからない場合
	 * @throws InvocationTargetException コンストラクタで例外がスローされた場合
	 * @throws IllegalAccessException コンストラクタにアクセスできない場合
	 * @throws InstantiationException クラスが <code>abstract</code> クラスを表す場合
	 */
	public static Object getInstance(
			final Constructor<?> constructor, final Object... param)
			throws InstantiationException, IllegalAccessException,
			InvocationTargetException, NoSuchMethodException {

		// オーバーロードメソッドをコール
		return getInstance(constructor.getDeclaringClass(), constructor, param);
	}

	/**
	 * <p>オブジェクトのインスタンスを取得。
	 * キャッシュを保持しており、キャッシュがあればそこから取得。</p>
	 * <p><var>param</var> に <code>null</code>
	 * が指定された場合や、複雑なコンストラクタ（基本型とラッパー型や、継承クラスだけが異なる）
	 * を持つクラスの場合、コンストラクタが識別できないため、動作は保証できない。<br>
	 * そういった場合は {@link #getInstance(Constructor, Object...)} を使用すること。</p>
	 * @param clazz オブジェクトのクラス
	 * @param param オブジェクトのコンストラクタへの引数
	 * @return オブジェクトのインスタンス
	 * @throws NullPointerException <var>clazz</var> が <code>null</code>
	 * @throws NoSuchMethodException 一致するコンストラクタが見つからない場合
	 * @throws InvocationTargetException コンストラクタで例外がスローされた場合
	 * @throws IllegalAccessException コンストラクタにアクセスできない場合
	 * @throws InstantiationException クラスが <code>abstract</code> クラスを表す場合
	 */
	public static Object getInstance(
			final Class<?> clazz, final Object... param)
			throws InstantiationException, IllegalAccessException,
			InvocationTargetException, NoSuchMethodException {

		// オーバーロードメソッドをコール
		return getInstance(clazz, null, param);
	}

	/**
	 * <p>オブジェクトのインスタンスを取得。
	 * キャッシュを保持しており、キャッシュがあればそこから取得。</p>
	 * <p><var>param</var> に <code>null</code>
	 * が指定された場合や、複雑なコンストラクタ（基本型とラッパー型や、継承クラスだけが異なる）
	 * を持つクラスの場合、コンストラクタが識別できないため、動作は保証できない。<br>
	 * そういった場合は {@link #getInstance(Constructor, Object...)} を使用すること。</p>
	 * @param className オブジェクトのクラス名
	 * @param param オブジェクトのコンストラクタへの引数
	 * @return オブジェクトのインスタンス
	 * @throws NullPointerException <var>className</var> が <code>null</code>
	 * @throws ClassNotFoundException <var>className</var> のクラスが存在しない場合
	 * @throws NoSuchMethodException 一致するコンストラクタが見つからない場合
	 * @throws InvocationTargetException コンストラクタで例外がスローされた場合
	 * @throws IllegalAccessException コンストラクタにアクセスできない場合
	 * @throws InstantiationException クラスが <code>abstract</code> クラスを表す場合
	 */
	public static Object getInstance(
			final String className, final Object... param)
			throws ClassNotFoundException, InstantiationException,
			IllegalAccessException, InvocationTargetException,
			NoSuchMethodException {

		// 引数チェック
		if (className == null) {
			throw new NullPointerException("className is null");
		}
		// オーバーロードメソッドをコール
		return getInstance(Class.forName(className), param);
	}

	/**
	 * 保持しているオブジェクトのキャッシュを解放する。
	 */
	public static synchronized void reset() {
		// キャッシュマップをクリアする
		caches.clear();
	}
}
