package common;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * オブジェクトやプロパティに関する処理を共通的に扱うためのユーティリティクラス。
 * @author Honeplus
 * @version $Id: ObjectUtility.java 32 2009-03-01 13:59:59Z Honeplus $
 */
public final class ObjectUtility {

	/**
	 * インスタンス作成抑止用コンストラクタ。
	 * @throws UnsupportedOperationException 実行不可
	 */
	private ObjectUtility() {
		throw new UnsupportedOperationException("this class is utility");
	}

	/**
	 * <p>渡されたパラメータから、動的にオブジェクトのインスタンスを作成。</p>
	 * <p><var>param</var> に <code>null</code>
	 * が指定された場合、コンストラクタが識別できない可能性がある。
	 * また、継承している親と子のクラスについては識別できない。
	 * 同様に基本型・ラッパー型の違いも識別できない。</p>
	 * @param type オブジェクトのクラス名
	 * @param param オブジェクトのコンストラクタへの引数
	 * @return オブジェクトのインスタンス
	 * @throws NoSuchMethodException 一致するコンストラクタが見つからない場合
	 * @throws InvocationTargetException コンストラクタで例外がスローされた場合
	 * @throws IllegalAccessException コンストラクタにアクセスできない場合
	 * @throws InstantiationException クラスが <code>abstract</code> クラスを表す場合
	 */
	public static Object newInstance(
			final Class<?> type, final Object... param)
			throws NoSuchMethodException, InvocationTargetException,
			IllegalAccessException, InstantiationException {

		// 引数チェック
		if (type == null) {
			throw new IllegalArgumentException("type is null");
		}
		// コンストラクタに渡すパラメータ定義を作成。
		// 生成したオブジェクトを返す
		return getConstructor(type, param).newInstance(param);
	}

	/**
	 * <p>指定されたパラメータに一致する、クラスのコンストラクタを探索。</p>
	 * <p><var>param</var> に <code>null</code>
	 * が指定された場合、コンストラクタが識別できない可能性がある。
	 * また、継承している親と子のクラスについては識別できない。
	 * 同様に基本型・ラッパー型の違いも識別できない。</p>
	 * @param type オブジェクトのクラス名
	 * @param param オブジェクトのコンストラクタへの引数
	 * @return 一致する <code>public</code> コンストラクタの <code>Method</code> オブジェクト
	 * @throws NoSuchMethodException 一致するコンストラクタが見つからない場合
	 * @throws NullPointerException <var>type</var> が <code>null</code>
	 */
	private static Constructor<?> getConstructor(final Class<?> type,
			final Object... param) throws NoSuchMethodException {

		// 渡されたパラメータのクラス定義を取得
		Class<?>[] paramTypes = null;
		boolean existedNull = false;
		StringBuilder b = new StringBuilder();	// 例外メッセージ用
		if (param != null) {
			paramTypes = new Class[param.length];
			for (int i = 0; i < param.length; i++) {
				if (i > 0) {
					b.append(',');
				}
				if (param[i] != null) {
					paramTypes[i] = param[i].getClass();
					b.append(paramTypes[i].getName());
				} else {
					paramTypes[i] = null;
					existedNull = true;
					b.append("null");
				}
			}
		}
		// とりあえず標準の単純な方法でコンストラクタを取得
		// ※ nullがあったら無理
		if (!existedNull) {
			try {
				return type.getConstructor(paramTypes);
			} catch (NoSuchMethodException e) {
				// この時点での例外は無視
				//e.printStackTrace();
			}
		}

		// 取れない場合、全コンストラクタの中に一致するものがあるか、独自に探索。
		// null やラッパー型が混ざっている場合、正しく判定できない可能性がある。
		// できる限り一致するコンストラクタを探索する。
		if (paramTypes != null) {
			// 全publicコンストラクタを取得
			Constructor<?>[] constructors = type.getConstructors();
			for (Constructor<?> con : constructors) {
				Class<?>[] declaredParamTypes = con.getParameterTypes();
				// パラメータ数の一致チェック
				if (declaredParamTypes.length != paramTypes.length) {
					continue;
				}
				// 型一致のチェック
				boolean available = true;
				for (int i = 0; i < declaredParamTypes.length; i++) {
					if (!isAvailable(declaredParamTypes[i], paramTypes[i])) {
						available = false;
						break;
					}
				}
				if (available) {
					// 全ての引数が利用可能であれば、そのコンストラクタを返す
					//TODO: 親子で判別しているコンストラクタがある場合、区別する対応が必要
					//TODO: 複数のコンストラクタに該当してしまう可能性が残っている
					return con;
				}
			}
		}
		// どれにも該当しない場合は、例外を投げる
		throw new NoSuchMethodException(
				"parameter mismatched : " + b.toString());
	}

	/** {@link #isAvailable(Class, Class)} の比較パターンを定義したマップ。 */
	private static final Map<Class<?>, Class<?>[]> AVAILABLE_PATTERN;
	static {
		Map<Class<?>, Class<?>[]> m = new HashMap<Class<?>, Class<?>[]>();
		// 基本型とラッパークラスの組み合わせと、暗黙の型変換
		// ※ ラッパー型の場合は暗黙の型変換は行えないが、パラメータについては
		//    この時点では見分けがつかないので、全て許容する。
		// int←int, Integer, byte, short, char
		m.put(int.class, new Class[]{int.class, Integer.class, byte.class,
			Byte.class, short.class, Short.class, char.class, Character.class});
		// Integer←int, Integer
		m.put(Integer.class, new Class[]{int.class, Integer.class});
		// long←long, Long, byte, short, int, char
		m.put(long.class, new Class[]{long.class, Long.class,
			Byte.class, short.class, Short.class, int.class, Integer.class,
			char.class, Character.class});
		// Long←long, Long
		m.put(Long.class, new Class[]{long.class, Long.class});
		// byte←byte, Byte
		m.put(byte.class, new Class[]{byte.class, Byte.class});
		// Byte←byte, Byte
		m.put(Byte.class, new Class[]{byte.class, Byte.class});
		// short←short, Short, byte
		m.put(short.class, new Class[]{
			short.class, Short.class, byte.class, Byte.class});
		// Short←short, Short
		m.put(Short.class, new Class[]{short.class, Short.class});
		// char←char, Character
		m.put(char.class, new Class[]{char.class, Character.class});
		// Character←char, Character
		m.put(Character.class, new Class[]{char.class, Character.class});
		// float←float, Float, byte, short, int, long, char
		m.put(float.class, new Class[]{float.class, Float.class, byte.class,
			Byte.class, short.class, Short.class, int.class, Integer.class,
			long.class, Long.class, char.class, Character.class});
		// Float←float, Float
		m.put(Float.class, new Class[]{float.class, Float.class});
		// double←double, Double, byte, short, int, long, char, float
		m.put(double.class, new Class[]{double.class, Double.class,
			byte.class, Byte.class, short.class, Short.class,
			int.class, Integer.class, long.class, Long.class,
			char.class, Character.class, float.class, Float.class});
		// Double←double, Double
		m.put(Double.class, new Class[]{double.class, Double.class});
		// boolean←boolean, Boolean
		m.put(boolean.class, new Class[]{boolean.class, Boolean.class});
		// Boolean←boolean, Boolean
		m.put(Boolean.class, new Class[]{boolean.class, Boolean.class});
		// 定数に格納
		AVAILABLE_PATTERN = Collections.unmodifiableMap(m);
	}

	/**
	 * 指定されたメソッド定義の型に、渡された型のパラメータが使用できるかを判定する。
	 * @param declared メソッド定義の型
	 * @param param パラメータの型
	 * @return <code>true</code> 利用可能、<code>false</code> 利用不可
     * @throws NullPointerException <var>declared</var> が <code>null</code>
	 */
	private static boolean isAvailable(
			final Class<?> declared, final Class<?> param) {

		// 値がnullの場合
		if (param == null) {
			if (declared.isPrimitive()) {
				// 基本型にnullは利用不可
				return false;
			}
			// それ以外は全てOK
			return true;
		}
		// パラメータの型を比較
		if (declared.isAssignableFrom(param)) {
			// 型が許可されたものならその時点でOK
			return true;
		}
		// 基本型とラッパークラス、暗黙の型変換の比較
		Class<?>[] pattern = AVAILABLE_PATTERN.get(declared);
		if (pattern != null) {
			for (Class<?> cl : pattern) {
				if (cl.isAssignableFrom(param)) {
					// メソッドが許容している型であればOK
					return true;
				}
			}
		}
		// 許可されていない組み合わせの場合、利用不可
		return false;
	}

	/**
	 * <p>プロパティファイル読み込み。</p>
	 * <p>ファイルは、{@link ClassLoader#getSystemResourceAsStream(String)}
	 * 経由で読み込む。</p>
	 * @param name プロパティファイル名
	 * @return プロパティ
     * @throws IOException プロパティファイル読み込み中にI/Oエラー
     * @throws IllegalArgumentException プロパティファイルのフォーマットが不正
     * @throws NullPointerException プロパティファイルがない
	 */
	public static Properties getProperties(final String name)
			throws IOException {

		// プロパティファイル読み込み
		Properties properties = new Properties();
		InputStream is = null;
		try {
			is = ClassLoader.getSystemResourceAsStream(name);
			properties.load(is);
		} finally {
			if (is != null) {
				is.close();
			}
		}
		return properties;
	}
}
