package common.client;

/**
 * <p>コンソールアプリ実装用の抽象クラス。</p>
 * <p>一部、デザインパターンの Template Method パターン風になっており、
 * 継承クラスでは {@link #validate(String[])}, {@link #parse(String[])},
 * {@link #run()}、そして <code>main</code>
 * の実装を行うことで、定型のコンソールアプリの作成が可能。<br>
 * <code>main</code> では、前述の3メソッドを表記の順番に実行する。</p>
 * @author Honeplus
 * @version $Id: AbstractConsole.java 24 2009-02-22 14:04:01Z Honeplus $
 */
public abstract class AbstractConsole {

	/**
	 * <p>アプリケーションを指定された終了コードで終了させる用通知する例外。</p>
	 * <p>そこらじゅうで {@link System#exit(int)}i してしまわないように定義。
	 * ただし、画面へのエラー表示などは、例外スロー前に済ませること。</p>
	 * @author Honeplus
	 * @version $Id: AbstractConsole.java 24 2009-02-22 14:04:01Z Honeplus $
	 */
	public static class SystemExitException extends Exception {

		/** {@link java.io.Serializable} で同一クラス定義を明示する一意なID */
		private static final long serialVersionUID = -4232255764264909472L;

		/** アプリケーションの終了コード（戻り値） */
		private ExitStatus status;

		/**
		 * アプリケーション終了時に返すべき終了コード（戻り値）を返す。
		 * @return 返すべき終了コード（戻り値）
		 */
		public ExitStatus getExitStatus() {
			return status;
		}

		/**
		 * 終了コードを設定して例外を作成する。
		 * @param exit 返すべき終了コード（戻り値）
		 */
		public SystemExitException(final ExitStatus exit) {
			// 終了コードを設定
			this.status = exit;
		}

		/**
		 * 終了コード、エラーメッセージを設定して例外を作成する。
		 * @param exit 返すべき終了コード（戻り値）
		 * @param message エラーメッセージ
		 */
		public SystemExitException(final ExitStatus exit,
				final String message) {
			// スーパークラスのコンストラクタをコール
			super(message);
			// 終了コードを設定
			this.status = exit;
		}

		/**
		 * 終了コード、エラー原因の例外を設定して例外を作成する。
		 * @param exit 返すべき終了コード（戻り値）
		 * @param cause エラー原因の例外
		 */
		public SystemExitException(final ExitStatus exit,
				final Throwable cause) {
			// スーパークラスのコンストラクタをコール
			super(cause);
			// 終了コードを設定
			this.status = exit;
		}

		/**
		 * 終了コード、エラーメッセージ、エラー原因を設定して例外を作成する。
		 * @param exit 戻すべき終了コード（戻り値）
		 * @param message エラーメッセージ
		 * @param cause エラー原因の例外
		 */
		public SystemExitException(final ExitStatus exit,
				final String message, final Throwable cause) {
			// スーパークラスのコンストラクタをコール
			super(message, cause);
			// 終了コードを設定
			this.status = exit;
		}
	}

	/**
	 * <p>コンソールから渡された引数のチェック（引数の有無・個数）。<p>
	 * <p><b>エラーの場合は、標準エラー出力にエラーを吐き、プログラム終了例外を投げる。</b></p>
	 * @param args 入力パラメータリスト
	 * @throws SystemExitException プログラムを終了させる必要がある場合
	 */
	protected void validate(final String[] args)
			throws SystemExitException {

		// 引数チェック
		if (args == null || args.length < 1) {
			// 引数なし時はヘルプメッセージ表示して終了
			printf(MessageUtility.getMessage("CONSOLE_HELP"));
			throw new SystemExitException(ExitStatus.EX_USAGE);
		}
	}

	/**
	 * <p>コンソールから渡された引数の解析、属性への保存。<p>
	 * <p><b>エラーの場合は、標準エラー出力にエラーを吐き、プログラム終了例外を投げる。</b></p>
	 * @param args 入力パラメータリスト
	 * @throws SystemExitException プログラムを終了させる必要がある場合
	 */
	protected abstract void parse(final String[] args)
			throws SystemExitException;

	/**
	 * <p>アプリケーション本体の実行。<p>
	 * <p><b>エラーの場合は、標準エラー出力にエラーを吐き、プログラム終了例外を投げる。</b></p>
	 * @throws SystemExitException プログラムを終了させる必要がある場合
	 */
	protected abstract void run()
			throws SystemExitException;

	/**
	 * <p>コンソールへの出力。</p>
	 * <p>コンソールがある場合のみ、{@link java.io.Console#printf(String, Object...)}
	 * を実行する。</p>
	 * <p>※「標準出力」ではないので注意。</p>
	 * @param format 書式文字列
	 * @param args 書式文字列内の書式指示子により参照される引数
	 */
	protected void printf(
			final String format, final Object... args) {
		if (System.console() != null) {
			System.console().printf(format, args);
		}
	}

	/**
	 * <p>コンソールからの実行用メイン実装補助メソッド。</p>
	 * <p><code>static</code> な <code>main</code> では継承クラスのクラス名が判らず、
	 * 継承クラスでの実装が必須となるため、Template Method
	 * パターン風に定型の処理を行うメソッドを定義する。<br>
	 * 継承クラスの <code>main</code> では、このメソッドを呼び出して簡易に実装が行える。</p>
	 * <p>指定されたクラスのインスタンスを作成し、
	 * {@link #validate(String[])}, {@link #parse(String[])}, {@link #run()}
	 * の3メソッドを順番に実行する。
	 * なお、クラスにはデフォルトコンストラクタが存在する必要がある。</p>
	 * @param clazz 実行するクラス
	 * @param args 入力パラメータリスト
	 */
	protected static void abstractMain(
			final Class<? extends AbstractConsole> clazz, final String[] args) {

		// 引数チェック
		if (clazz == null) {
			throw new IllegalArgumentException("clazz is null");
		}

		// 指定されたオプションに従い文字列変換を実施
		ExitStatus status = ExitStatus.EX_OK;
		try {
			AbstractConsole con = clazz.newInstance();
			// 引数チェック（引数の有無・個数）
			con.validate(args);
			// 引数の解析、属性への格納
			con.parse(args);
			// 処理の実行
			con.run();
		} catch (SystemExitException e) {
			// プログラムを指定された終了コードで終了させる例外
			status = e.getExitStatus();
		} catch (Throwable e) {
			// 想定外のエラーが発生
			System.err.println(MessageUtility.getMessage(
					"CONSOLE_EXCEPTION",
					MessageUtility.getPrintStackTrace(e)));
			status = ExitStatus.EX_SOFTWARE;
		}
		// 終了コードを返し、プログラムを終了
		System.exit(status.intValue());
	}
}
