package common.client;

/**
 * <p>アプリケーションの終了コード（戻り値）を表す列挙値。</p>
 * <p>{@link System#exit(int)} に渡されることを想定。</p>
 * <p><a href="http://www.jp.freebsd.org/cgi/mroff.cgi?sect=3&cmd=&lc=1&subdir=man&dir=jpman-6.0.0%2Fman&subdir=man&man=sysexits">sysexits</a>
 * の定義を参照させていただいています。</p>
 * @author Honeplus
 * @version $Id: ExitStatus.java 23 2009-02-21 18:26:34Z Honeplus $
 */
public enum ExitStatus {

	/**
	 * 正常終了。
	 */
	EX_OK(0),

	/**
	 * コマンドが不正に使用されました。
	 * 例えば、引数の数の誤り、誤ったフラグ、パラメータの誤った構文など。
	 */
	EX_USAGE(65),

	/**
	 * 入力データがどこかで不正です。
	 * これは、ユーザデータのみに使用され、システムファイルには使用すべきではありません。
	 */
	EX_DATAERR(65),

	/**
	 * 入力ファイル（システムファイルではなく）が存在しないか、
	 * または読み込み可能ではありません。
	 * これには、メーラの ``No message'' のようなエラーも含まれます
	 * （このようなエラーを捕えたいなら）。
	 */
	EX_NOINPUT(66),

	/**
	 * 指定されたユーザが存在しません。
	 * これは、メールアドレスやリモートログインに使用されるかもしれません。
	 */
	EX_NOUSER(67),

	/**
	 * 指定されたホストが存在しません。
	 * これは、メールアドレスやネットワークリクエストに使われます。
	 */
	EX_NOHOST(68),

	/**
	 * そのサービスは利用できません。
	 * これは、サポートプログラムやファイルが存在しない場合に起きます。
	 * これは、また、理由は知らないけれども、何かやりたいことができないときに、
	 * 雑多なメッセージとして使うことができます。
	 */
	EX_UNAVAILABLE(69),

	/**
	 * 内部ソフトウェアエラーが検出されました。
	 * これは、できるだけオペレーティングシステム関連ではないエラーに限るべきです。
	 */
	EX_SOFTWARE(70),

	/**
	 * オペレーティングシステムエラーが検出されました。
	 * これは、``cannot fork''（フォークできない）
	 * ``cannot create pipe''（パイプを作成できない）などのような
	 * エラーに使うよう意図されています。
	 * これには、 passwd ファイルに存在しないユーザに返される
	 * getuid ようなものも含まれます。
	 */
	EX_OSERR(71),

	/**
	 * システムファイルのどれか（例えば、/etc/passwd 、/var/run/utmp など）が存在しないか、
	 * オープンできないか、または何らかのエラー（例えば、構文エラー）があります。
	 */
	EX_OSFILE(72),

	/**
	 * （ユーザ指定の）出力ファイルが作成できません。
	 */
	EX_CANTCREAT(73),

	/**
	 * いずれかのファイルで I/O 操作中にエラーが起きました。
	 */
	EX_IOERR(74),

	/**
	 * 一時的な失敗で、本当のエラーではないことを示します。
	 * sendmail では、これはメーラが（例えば）接続を確立できず、
	 * リクエストは後で再試行すべきであることを意味します。
	 */
	EX_TEMPFAIL(75),

	/**
	 * リモートシステムがプロトコル交換の間に ``not possible''（可能でない）何かを返しました。
	 */
	EX_PROTOCOL(76),

	/**
	 * 操作を実行するための十分な許可を持っていません。
	 * これは、 {@link #EX_NOINPUT} または {@link #EX_CANTCREAT}
	 * を使用するべきファイルシステムの問題を対象にしていません。
	 * より高いレベルの許可を対象にしています。
	 */
	EX_NOPERM(77),

	/**
	 * 何かコンフィギュレーションされていないもの、
	 * またはコンフィギュレーションが間違っている状態が見つかった。
	 */
	EX_CONFIG(78);

	/** 終了コード */
	private int exitCode;

	/**
	 * 終了コード値を設定する。
	 * @param code 終了コード
	 */
	private ExitStatus(final int code) {
		exitCode = code;
	}

	/**
	 * 各定義の終了コードを返す。
	 * @return 終了コード
	 */
	public int intValue() {
		return exitCode;
	}
}
