package common.client;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ResourceBundle;

/**
 * <p>メッセージ処理を共通的に扱うためのユーティリティクラス。</p>
 * <p>単純なアプリケーション用。
 * 複雑なアプリケーションに用いる場合、他のリソースも含めて管理することを検討。</p>
 * @author Honeplus
 * @version $Id: MessageUtility.java 23 2009-02-21 18:26:34Z Honeplus $
 */
public final class MessageUtility {

	/** メッセージを格納したプロパティファイル : <code>message</code> */
	private static final String MESSAGE_PEROPERTIES = "message";

	/**
	 * インスタンス作成抑止用コンストラクタ。
	 * @throws UnsupportedOperationException 実行不可
	 */
	private MessageUtility() {
		throw new UnsupportedOperationException("this class is utility");
	}

	/**
	 * <p>メッセージプロパティから指定されたキーの値を読み込む。</p>
	 * <p>メッセージの取得には、{@link ResourceBundle} を用いており、
	 * メッセージの言語設定による切り替えに対応する。</p>
	 * @param key メッセージリソースキー値
	 * @return メッセージ
	 */
	public static String getMessage(final String key) {
		return ResourceBundle.getBundle(MESSAGE_PEROPERTIES).getString(key);
	}

	/**
	 * <p>メッセージプロパティから指定されたキーの値を読み込む。</p>
	 * <p>メッセージの取得には、{@link ResourceBundle} を用いており、
	 * メッセージの言語設定による切り替えに対応する。<br>
	 * {@link String#format(String, Object...)}
	 * のオブジェクトをまとめて指定可能。</p>
	 * @param key メッセージリソースキー値
	 * @param args 書式文字列の書式指示子により参照される引数。書式指示子よりも引数が多い場合、余分な引数は無視される。
	 * @return メッセージ
	 */
	public static String getMessage(final String key, final Object ... args) {
		return String.format(getMessage(key), args);
	}

	/**
	 * 例外の {@link Throwable#printStackTrace()} を文字列として取得する。
	 * @param e 例外
	 * @return printStackTrace の結果
	 * @throws NullPointerException <var>e</var> が <code>null</code>
	 */
	public static String getPrintStackTrace(final Throwable e) {
		StringWriter s = new StringWriter();
		e.printStackTrace(new PrintWriter(s));
		return s.toString();
	}
}
