package converter.client;

import common.client.AbstractConsole;
import common.client.ExitStatus;
import common.client.MessageUtility;
import converter.text.TextConvertUtility;

/**
 * <p>文字列変換処理ユーティリティ コンソール。</p>
 * <p>※ このアプリは、ライブラリとして使うためのユーティリティ。
 *       そのため、このコンソールはほとんど実験用。
 *       まじめに使うなら getOpt などの導入を検討。</p>
 * @author Honeplus
 * @version $Id: TextConvertConsole.java 31 2009-03-01 12:37:38Z Honeplus $
 */
public final class TextConvertConsole extends AbstractConsole {

	/** 変換処理に指定する処理内容 */
	private TextConvertUtility.RunType runType;
	/** 変換処理に指定する変換対象の文字列 */
	private String text;

	/**
	 * <p>コンソールから渡された引数のチェック（引数の有無・個数）。<p>
	 * <p><b>エラーの場合は、標準エラー出力にエラーを吐き、プログラムを終了する。</b></p>
	 * @param args 入力パラメータリスト
	 * @throws SystemExitException プログラムを終了させる必要がある場合
	 */
	@Override
	protected void validate(final String[] args)
			throws SystemExitException {
		// スーパークラスの引数チェック（null or length < 1）を実施
		super.validate(args);
		// 追加の引数チェック
		if (args.length < 2) {
			// パラメータが不足時は、エラーを表示して終了
			System.err.println(MessageUtility.getMessage(
					"CONSOLE_ILLEGAL_ARRGUMENT", args[0]));
			throw new SystemExitException(ExitStatus.EX_USAGE);
		}
	}

	/**
	 * <p>コンソールから渡された引数の解析、属性への保存。<p>
	 * <p><b>エラーの場合は、標準エラー出力にエラーを吐き、プログラム終了例外を投げる。</b></p>
	 * @param args 入力パラメータリスト
	 * @throws SystemExitException プログラムを終了させる必要がある場合
	 */
	@Override
	protected void parse(final String[] args)
			throws SystemExitException {

		// オプション解析
		if (args[0].matches("[-/][Ff]")) {
			// 半角を全角に変換
			this.runType = TextConvertUtility.RunType.HALFWIDTH_TO_FULLWIDTH;
		} else if (args[0].matches("[-/][Hh]")) {
			// 全角を半角に変換
			this.runType = TextConvertUtility.RunType.FULLWIDTH_TO_HALFWIDTH;
		} else if (args[0].matches("[-/].*")) {
			// オプションが指定されたが不正
			System.err.println(MessageUtility.getMessage(
					"CONSOLE_ILLEGAL_OPTION", args[0]));
			throw new SystemExitException(ExitStatus.EX_USAGE);
		} else {
			// オプションが設定されていない
			System.err.println(MessageUtility.getMessage(
					"CONSOLE_NOTFOUND_OPTION", args[0]));
			throw new SystemExitException(ExitStatus.EX_USAGE);
		}
		// 文字列が誤ってスペース区切りになっていることを想定して、
		// 一連の文字列に結合。
		StringBuilder b = new StringBuilder(args[1]);
		for (int i = 2; i < args.length; i++) {
			b.append(' ');
			b.append(args[i]);
		}
		text = b.toString();
	}

	/**
	 * <p>アプリケーション本体の実行。<p>
	 * <p><b>エラーの場合は、標準エラー出力にエラーを吐き、プログラム終了例外を投げる。</b></p>
	 */
	@Override
	protected void run() {
		// 文字列の変換を行い、結果を標準出力に出力
		System.out.println(
				TextConvertUtility.convert(this.text, this.runType));
	}

	/**
	 * コンソールからの実行用メイン。
	 * @param args 入力パラメータリスト
	 */
	public static void main(final String[] args) {
		// validate(), parse(), run() が順番に実行される
		abstractMain(TextConvertConsole.class, args);
	}
}
