package converter.text;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * マッピングに従い、指定されたパターンの変換を行うためのクラス。
 * @author Honeplus
 * @version $Id: MappingConverter.java 31 2009-03-01 12:37:38Z Honeplus $
 */
public class MappingConverter implements TextConverter {

	/** マッピングパターンマップ */
	private Map<String, String> mapping;

	/**
	 * <p>指定されたマップをマッピングパターンとして読み込み、
	 * コンバーターを作成する。</p>
	 * <p>マップは変換処理で使用するため内部で変換する。</p>
	 * @param map マッピングパターン
     * @throws IllegalArgumentException <var>map</var> が <code>null</code>
	 */
	public MappingConverter(final Map<String, String> map) {
		// マップ変換ありでオーバーロードメソッドをコール
		this(map, false);
	}

	/**
	 * <p>指定されたマップをマッピングパターンとして読み込み、
	 * コンバーターを作成する。</p>
	 * <p>渡されたマップは、<var>directed</var> に <code>true</code>
	 * が指定された場合そのまま、<code>false</code>
	 * が指定された場合は内部で変換して、変換処理で使用する。</p>
	 * @param map マッピングパターン
	 * @param directed <code>true</code> マップをそのまま使用
     * @throws IllegalArgumentException <var>map</var> が <code>null</code>
	 */
	public MappingConverter(
			final Map<String, String> map, final boolean directed) {
		// 指定されたマップをマッピングパターンとして読み込む
		setMapping(map, directed);
	}

	/**
	 * <p>空のコンバーターを作成する（継承クラス実装用）。</p>
	 * <p>継承クラスでは、必ず {@link #setMapping} により
	 * {@link #mapping} を設定すること。</p>
	 */
	protected MappingConverter() {
	}

	/**
	 * マッピングパターンマップを取得。
	 * @return マッピングパターン
	 */
	protected Map<String, String> getMapping() {
		return mapping;
	}

	/**
	 * マッピングパターンマップを設定。
	 * @param map マッピングパターン
	 * @param directed <code>true</code> マップをそのまま使用
     * @throws IllegalArgumentException <var>map</var> が <code>null</code>
	 */
	protected void setMapping(
			final Map<String, String> map, final boolean directed) {
		// 引数チェック
		if (map == null) {
			throw new IllegalArgumentException("map is null");
		}
		// 引数に応じて、マップを変換する
		if (!directed) {
			// 文字列長に応じてマップをソート
			this.mapping = getLengthOrderMap(map);
		} else {
			// マップをそのまま使用する
			this.mapping = map;
		}
	}

	/**
	 * マッピングパターンマップを設定。
	 * @param map マッピングパターン
     * @throws IllegalArgumentException <var>map</var> が <code>null</code>
	 */
	protected void setMapping(final Map<String, String> map) {
		// マップ変換ありでオーバーロードメソッドをコール
		setMapping(map, false);
	}

	/**
	 * 渡されたテキストに対して、指定されたパターンによる変換を行う。
	 * @param s 変換元テキスト
	 * @return 変換後テキスト
	 */
	@Override
	public String convert(final String s) {
		// null値チェック
		if (s == null) {
			return null;	// nullはそのまま
		}
		// マッピングパターンより文字列の変換を行う
		StringBuilder b = new StringBuilder(s);
		for (Map.Entry<String, String> m : mapping.entrySet()) {
			int i;
			while ((i = b.indexOf(m.getKey())) > -1) {
				b.replace(i, i + m.getKey().length(), m.getValue());
			}
		}
		return b.toString();
	}

	/**
	 * <p>渡された文字に対して、指定されたパターンによる変換を行う。</p>
	 * <p>コンストラクタで、1文字→1文字の変換パターンのみを記述した
	 * プロパティファイルが指定されている必要がある。
	 * 2文字以上または0文字への変換パターンが存在する場合、
	 * パターン該当時に {@link UnsupportedOperationException} を投げる。</p>
	 * @param c 変換元文字
	 * @return 変換後文字
	 * @throws UnsupportedOperationException 1文字変換をサポートしないプロパティファイル
	 */
	@Override
	public char convert(final char c) {
		// マッピングパターンより文字の変換を行う
		String m = mapping.get(String.valueOf(c));
		char result = c;
		if (m != null) {
			// 1文字→1文字以外の変換が定義されている場合、例外を投げる
			if (m.length() != 1) {
				throw new UnsupportedOperationException(
						"unsupported properties : \"" + c + "\"=\"" + m + "\"");
			}
			result = m.charAt(0);
		}
		return result;
	}

	/**
	 * <p>指定された文字列マップを、文字列長の降順でソートする。</p>
	 * <p>短い文字が長い文字より先にパターンに該当してしまう可能性があるので、
	 * その対策用。</p>
	 * @param map ソートもとのマップ
	 * @return 文字列長の降順でソートされたマップ
	 */
	private LinkedHashMap<String, String> getLengthOrderMap(
			final Map<String, String> map) {

		// 引数チェック
		if (map == null) {
			return null;
		}
		// 文字列長順にソート用のマップに格納しなおす。
		TreeMap<Integer, List<Map.Entry<String, String>>> sort
			= new TreeMap<Integer, List<Map.Entry<String, String>>>();
		for (Map.Entry<String, String> m : map.entrySet()) {
			int length = m.getKey().length();
			List<Map.Entry<String, String>> list = sort.get(length);
			if (list == null) {
				list = new ArrayList<Map.Entry<String, String>>();
				sort.put(length, list);
			}
			list.add(m);
		}
		// ソートされた順番で、並び順を保持するマップに格納しなおす。
		LinkedHashMap<String, String> link
			= new LinkedHashMap<String, String>();
		for (Map.Entry<Integer, List<Map.Entry<String, String>>> m
				: sort.descendingMap().entrySet()) {
			for (Map.Entry<String, String> l :  m.getValue()) {
				link.put(l.getKey(), l.getValue());
			}
		}
		return link;
	}
}
