package converter.text;

import java.util.Map;

/**
 * マッピングに従い、指定された正規表現パターンでの変換を行うためのクラス。
 * @author Honeplus
 * @version $Id: MappingRegexConverter.java 32 2009-03-01 13:59:59Z Honeplus $
 */
public class MappingRegexConverter extends MappingConverter {

	/**
	 * <p>指定されたマップをマッピングパターンとして読み込み、
	 * コンバーターを作成する。</p>
	 * <p>渡されたマップは、そのまま変換処理で使用する。
	 * 処理順序を考慮する場合は、{@link java.util.LinkedHashMap} 等を指定する。</p>
	 * @param map マッピングパターン
     * @throws IllegalArgumentException <var>map</var> が <code>null</code>
	 */
	public MappingRegexConverter(final Map<String, String> map) {
		// 指定されたマップをマッピングパターンとして読み込む
		super(map, true);
	}

	/**
	 * <p>空のコンバーターを作成する（継承クラス実装用）。</p>
	 * <p>継承クラスでは、必ず {@link #setMapping} により
	 * {@link #mapping} を設定すること。</p>
	 */
	protected MappingRegexConverter() {
	}

	/**
	 * <p>渡されたテキストに対して、指定された正規表現パターンによる変換を行う。</p>
	 * <p>汎用の処理を行っているため、実行速度は遅め。</p>
	 * @param s 変換元テキスト
	 * @return 変換後テキスト
	 */
	@Override
	public String convert(final String s) {
		// null値チェック
		if (s == null) {
			return null;	// nullはそのまま
		}
		// マッピングパターンより文字列の変換を行う
		String r = s;
		for (Map.Entry<String, String> m : getMapping().entrySet()) {
			r = r.replaceAll(m.getKey(), m.getValue());
		}
		return r;
	}

	/**
	 * <p>（<b>未サポート</b>）1文字変換、。</p>
	 * <p>正規表現では1文字単位の処理は不可能。</p>
	 * @param c 変換元文字
	 * @return 変換後文字
	 * @throws UnsupportedOperationException 未サポート
	 */
	@Override
	public char convert(final char c) {
		throw new UnsupportedOperationException("not support");
	}
}
