package converter.text;

import java.util.ArrayList;
import java.util.List;

import common.ObjectPool;
import common.ObjectUtility;

/**
 * 文字列変換処理をまとめたユーティリティクラス。
 * @author Honeplus
 * @version $Id: TextConvertUtility.java 32 2009-03-01 13:59:59Z Honeplus $
 */
public final class TextConvertUtility {

	/**
	 * 行う変換処理を指定するための列挙値。
	 * @author Honeplus
	 * @version $Id: TextConvertUtility.java 32 2009-03-01 13:59:59Z Honeplus $
	 */
	public enum RunType {
		/** 全角英字→半角英字 */
		FULLWIDTH_LATIN_TO_HALFWIDTH,
		/** 全角数字→半角数字 */
		FULLWIDTH_NUMBER_TO_HALFWIDTH,
		/** 全角英数→半角英数 */
		FULLWIDTH_LATIN_AND_NUMBER_TO_HALFWIDTH,
		/**
		 * 全角記号→半角記号。<br>
		 * 対象とする記号については、{@link #FULLWIDTH_MARK_TO_HALFWIDTH_PROPERTIES} を参照。
		 */
		FULLWIDTH_MARK_TO_HALFWIDTH,
		/**
		 * 全角英数記号→半角英数記号。<br>
		 * 対象とする記号は {@link #FULLWIDTH_MARK_TO_HALFWIDTH} を参照。
		 */
		FULLWIDTH_LATIN_AND_NUMBER_AND_MARK_TO_HALFWIDTH,
		/** 全角カタカナ→半角カタカナ */
		FULLWIDTH_KANA_TO_HALFWIDTH,
		/** 全角→半角（可能なもの全て） */
		FULLWIDTH_TO_HALFWIDTH,

		/** 半角英字→全角英字 */
		HALFWIDTH_LATIN_TO_FULLWIDTH,
		/** 半角数字→全角数字 */
		HALFWIDTH_NUMBER_TO_FULLWIDTH,
		/** 半角英数→全角英数 */
		HALFWIDTH_LATIN_AND_NUMBER_TO_FULLWIDTH,
		/**
		 * 半角記号→全角記号。<br>
		 * 対象とする記号については、{@link #HALFWIDTH_MARK_TO_FULLWIDTH_PROPERTIES} を参照。
		 */
		HALFWIDTH_MARK_TO_FULLWIDTH,
		/**
		 * 半角英数記号→全角英数記号。<br>
		 * 対象とする記号は {@link #HALFWIDTH_MARK_TO_FULLWIDTH} を参照。
		 */
		HALFWIDTH_LATIN_AND_NUMBER_AND_MARK_TO_FULLWIDTH,
		/** 半角カタカナ→全角カタカナ */
		HALFWIDTH_KANA_TO_FULLWIDTH,
		/** 半角→全角（可能なもの全て） */
		HALFWIDTH_TO_FULLWIDTH,

		/** ひらがな→カタカナ */
		HIRAGANA_TO_KATAKANA,
		/** カタカナ→ひらがな */
		KATAKANA_TO_HIRAGANA,

		/** ローマ数字→似た半角英字文字列（IやV等への変換） */
		ROMAN_NUMERALS_TO_HALFWIDTH,
		/** 丸囲み数字→似た半角英数文字列（(1)等への変換） */
		ENCIRCLED_NUMBER_HALFWIDTH,
		/** ローマ数字・丸数字→似た半角英数文字列（Iや(1)への変換） */
		ROMAN_AND_ENCIRCLED_HALFWIDTH,
	}

	/**
	 * 全角記号→半角記号変換用のマッピング定義ファイル
	 *  : <code>converter/text/fullwidthMark2halfwidth.properties</code>
	 */
	private static final String FULLWIDTH_MARK_TO_HALFWIDTH_PROPERTIES
		= "converter/text/fullwidthMark2halfwidth.properties";
	/**
	 * 半角記号→全角記号変換用のマッピング定義ファイル
	 *  : <code>converter/text/halfwidthMark2fullwidth.properties</code>
	 */
	private static final String HALFWIDTH_MARK_TO_FULLWIDTH_PROPERTIES
		= "converter/text/halfwidthMark2fullwidth.properties";
	/**
	 * 全角カナ→半角カナ変換用のマッピング定義ファイル
	 *  : <code>converter/text/fullwidthKana2halfwidth.properties</code>
	 */
	private static final String FULLWIDTH_KANA_TO_HALFWIDTH_PROPERTIES
		= "converter/text/fullwidthKana2halfwidth.properties";
	/**
	 * ローマ数字→似た半角英字文字列変換用のマッピング定義ファイル
	 *  : <code>converter/text/fullwidthKana2halfwidth.properties</code>
	 */
	private static final String ROMAN_NUMERALS_TO_HALFWIDTH_PROPERTIES
		= "converter/text/romanNumerals2halfwidth.properties";
	/**
	 * 丸囲み数字→似た半角英数文字列変換用のマッピング定義ファイル
	 *  : <code>converter/text/fullwidthKana2halfwidth.properties</code>
	 */
	private static final String ENCIRCLED_NUMBER_HALFWIDTH_PROPERTIES
		= "converter/text/encircledNumber2halfwidth.properties";

	/**
	 * インスタンス作成抑止用コンストラクタ。
	 * @throws UnsupportedOperationException 実行不可
	 */
	private TextConvertUtility() {
		throw new UnsupportedOperationException("this class is utility");
	}

	/**
	 * 指定された処理の文字列変換を実施する。
	 * @param text 変換元文字列
	 * @param runType 変換処理
	 * @return 変換後文字列
	 */
	public static String convert(final String text, final RunType runType) {

		String result = text;
		// 指定された変換処理を実施し、その結果を返す。
		// 1文字ずつまとめて実行することが可能なものは、Converterを集めて
		// 後でまとめて実行する。
		List<TextConverter> convList = new ArrayList<TextConverter>();
		switch (runType) {
		case FULLWIDTH_LATIN_TO_HALFWIDTH:
			// 全角英字→半角英字
			convList.add(getInstance(FullwidthLatin2Halfwidth.class));
			break;
		case FULLWIDTH_NUMBER_TO_HALFWIDTH:
			// 全角数字→半角数字
			convList.add(getInstance(FullwidthNumber2Halfwidth.class));
			break;
		case FULLWIDTH_LATIN_AND_NUMBER_TO_HALFWIDTH:
			// 全角英数→半角英数
			convList.add(getInstance(FullwidthLatin2Halfwidth.class));
			convList.add(getInstance(FullwidthNumber2Halfwidth.class));
			break;
		case FULLWIDTH_MARK_TO_HALFWIDTH:
			// 全角記号→半角記号
			convList.add(getInstance(PropertiesMappingConverter.class,
					FULLWIDTH_MARK_TO_HALFWIDTH_PROPERTIES));
			break;
		case FULLWIDTH_LATIN_AND_NUMBER_AND_MARK_TO_HALFWIDTH:
			// 全角英数記号→半角英数記号
			convList.add(getInstance(FullwidthLatin2Halfwidth.class));
			convList.add(getInstance(FullwidthNumber2Halfwidth.class));
			convList.add(getInstance(PropertiesMappingConverter.class,
					FULLWIDTH_MARK_TO_HALFWIDTH_PROPERTIES));
			break;
		case FULLWIDTH_KANA_TO_HALFWIDTH:
			// 全角カタカナ→半角カタカナ
			// ※ カタカナは濁点で文字数が変動するので、別途処理
			result = getInstance(PropertiesMappingConverter.class,
					FULLWIDTH_KANA_TO_HALFWIDTH_PROPERTIES).convert(result);
			break;
		case FULLWIDTH_TO_HALFWIDTH:
			// 全角→半角（可能なもの全て）
			convList.add(getInstance(FullwidthLatin2Halfwidth.class));
			convList.add(getInstance(FullwidthNumber2Halfwidth.class));
			convList.add(getInstance(PropertiesMappingConverter.class,
					FULLWIDTH_MARK_TO_HALFWIDTH_PROPERTIES));
			// ※ カタカナは濁点で文字数が変動するので、別途処理
			result = (getInstance(PropertiesMappingConverter.class,
					FULLWIDTH_KANA_TO_HALFWIDTH_PROPERTIES)).convert(result);
			break;
		case HALFWIDTH_LATIN_TO_FULLWIDTH:
			// 半角英字→全角英字
			convList.add(getInstance(HalfwidthLatin2Fullwidth.class));
			break;
		case HALFWIDTH_NUMBER_TO_FULLWIDTH:
			// 半角数字→全角数字
			convList.add(getInstance(HalfwidthNumber2Fullwidth.class));
			break;
		case HALFWIDTH_LATIN_AND_NUMBER_TO_FULLWIDTH:
			// 半角英数→全角英数
			convList.add(getInstance(HalfwidthLatin2Fullwidth.class));
			convList.add(getInstance(HalfwidthNumber2Fullwidth.class));
			break;
		case HALFWIDTH_MARK_TO_FULLWIDTH:
			// 半角記号→全角記号
			convList.add(getInstance(PropertiesMappingConverter.class,
					HALFWIDTH_MARK_TO_FULLWIDTH_PROPERTIES));
			break;
		case HALFWIDTH_LATIN_AND_NUMBER_AND_MARK_TO_FULLWIDTH:
			// 半角英数記号→全角英数記号
			convList.add(getInstance(HalfwidthLatin2Fullwidth.class));
			convList.add(getInstance(HalfwidthNumber2Fullwidth.class));
			convList.add(getInstance(PropertiesMappingConverter.class,
					HALFWIDTH_MARK_TO_FULLWIDTH_PROPERTIES));
			break;
		case HALFWIDTH_KANA_TO_FULLWIDTH:
			// 半角カタカナ→全角カタカナ
			// ※ カタカナは濁点で文字数が変動するので、別途処理
			result = getInstance(HalfwidthKana2Fullwidth.class)
				.convert(result);
			break;
		case HALFWIDTH_TO_FULLWIDTH:
			// 半角→全角（可能なもの全て）
			convList.add(getInstance(HalfwidthLatin2Fullwidth.class));
			convList.add(getInstance(HalfwidthNumber2Fullwidth.class));
			convList.add(getInstance(PropertiesMappingConverter.class,
					HALFWIDTH_MARK_TO_FULLWIDTH_PROPERTIES));
			// ※ カタカナは濁点で文字数が変動するので、別途処理
			result = getInstance(HalfwidthKana2Fullwidth.class)
				.convert(result);
			break;
		case HIRAGANA_TO_KATAKANA:
			// ひらがな→カタカナ
			result = getInstance(Hiragana2Katakana.class)
				.convert(result);
			break;
		case KATAKANA_TO_HIRAGANA:
			// カタカナ→ひらがな
			result = getInstance(Katakana2Hiragana.class)
				.convert(result);
			break;
		case ROMAN_NUMERALS_TO_HALFWIDTH:
			// ローマ数字→似た半角英字文字列（IやV等への変換）
			// ※ 文字数が変動するので、別途処理
			result = (getInstance(PropertiesMappingConverter.class,
					ROMAN_NUMERALS_TO_HALFWIDTH_PROPERTIES)).convert(result);
			break;
		case ENCIRCLED_NUMBER_HALFWIDTH:
			// 丸囲み数字→似た半角英数文字列（(1)等への変換）
			// ※ 文字数が変動するので、別途処理
			result = (getInstance(PropertiesMappingConverter.class,
					ENCIRCLED_NUMBER_HALFWIDTH_PROPERTIES)).convert(result);
			break;
		case ROMAN_AND_ENCIRCLED_HALFWIDTH:
			// ローマ数字・丸数字→似た半角英数文字列（Iや(1)への変換）
			// ※ いずれも文字数が変動するので、別途処理
			result = (getInstance(PropertiesMappingConverter.class,
					ROMAN_NUMERALS_TO_HALFWIDTH_PROPERTIES)).convert(result);
			result = (getInstance(PropertiesMappingConverter.class,
					ENCIRCLED_NUMBER_HALFWIDTH_PROPERTIES)).convert(result);
			break;
		default:
			throw new UnsupportedOperationException("unpopulated runType");
		}

		// まとめて変換処理を実行し、その結果を返す
		return convert(result, convList);
	}

	/**
	 * <p>{@link TextConverter} のインスタンスを取得。
	 * {@link ObjectPool} にキャッシュを保持しており、キャッシュがあればそこから取得。</p>
	 * <p><var>param</var> に <code>null</code>
	 * が指定された場合や、複雑なコンストラクタ（基本型とラッパー型や、継承クラスだけが異なる）
	 * を持つクラスの場合、コンストラクタが識別できないため、動作は保証できない。</p>
	 * @param clazz オブジェクトのクラス名
	 * @param param オブジェクトのコンストラクタへの引数
	 * @return オブジェクトのインスタンス
	 * @throws RuntimeException オブジェクトの作成失敗
	 */
	private static TextConverter getInstance(
			final Class<? extends TextConverter> clazz, final Object... param) {

		try {
			// PropertiesMappingConverter系のクラスはnewが重いので、
			// キャッシュがあればキャッシュから取得する。
			if (PropertiesMappingConverter.class.isAssignableFrom(clazz)
					|| PropertiesMappingRegexConverter.class
						.isAssignableFrom(clazz)) {
				return (TextConverter) ObjectPool.getInstance(clazz, param);
			}
			// それ以外の小さな変換クラスは、キャッシュ処理のオーバーヘッドで
			// 遅くなるので、自分でnewする。
			return (TextConverter) ObjectUtility.newInstance(clazz, param);
		} catch (Exception e) {
			// NoSuchMethodException 等によりオブジェクトの作成に
			// 失敗した場合は、RuntimeException に詰め替えて返す
			// （通常はクラス記述ミスなどのコーディングミスであるため）
			throw new RuntimeException(e);
		}
	}

	/**
	 * 文字列に対して、1文字ずつの変換 <var>convList</var> を実施。
	 * @param s 変換元文字列
	 * @param convList 実行変換処理コンバーター
	 * @return 変換後文字列
	 * @throws NullPointerException <var>convList</var> が <code>null</code>
	 */
	private static String convert(
			final String s, final List<TextConverter> convList) {

		// null値・変換無しチェック
		if (s == null || convList.isEmpty()) {
			return s;	// nullはそのまま
		}
		// はじめに全文字分の領域を確保し、変換する文字だけを置き換え
		StringBuilder b = new StringBuilder(s);
		for (int i = 0; i < b.length(); i++) {
			char c = b.charAt(i);
			for (TextConverter conv : convList) {
				c = conv.convert(c);
			}
			b.setCharAt(i, c);
		}
		return b.toString();
	}
}
