package common;

import static org.junit.Assert.fail;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

/**
 * <p>JUnitでのテスト用に <code>private, protected</code>
 * といった外部からアクセスできないメソッドを実行するためのクラス。
 * クラス・オブジェクトに被せるように使用する。</p><ul>
 * 
 * <li><p>単純にメソッドをテストするだけの場合は、下記のように記述する。</p>
 * <p>例、<samp>Foo</samp> クラスの <samp>String convert(String)</samp>
 * メソッドで戻り値を確認する。</p>
 * <pre><samp> HiddenAccessor acc = new HiddenAccessor(Foo.class);
 * acc.setMethod("convert", String.class);
 * assertEquals("output-string", acc.invoke("input-string"));</samp></pre></li>
 * 
 * <li><p>コンストラクタをテストする場合は、下記のように記述する。</p>
 * <p>例、<samp>Bar</samp> クラスの <samp>Bar(int)</samp> コンストラクタ。
 *     引数のパラメータが生成されたオブジェクトの <samp>num</samp>
 *     フィールドに格納されることを確認する。</p>
 * <pre><samp> HiddenAccessor acc = new HiddenAccessor(Bar.class);
 * acc.setConstructor(int.class);
 * acc.newInstance(10);
 * assertEquals(10, acc.getField("num"));</samp></pre>
 * <p><samp>num</samp> が <samp>getNum()</samp>
 * 等で普通に参照できるのであれば、当然下記でも問題ない。</p>
 * <pre><samp> HiddenAccessor acc = new HiddenAccessor(Bar.class);
 * acc.setConstructor(int.class);
 * assertEquals(10, ((Bar) acc.newInstance(10)).getNum());</samp></pre></li>
 * 
 * <li><p>事前にパラメータの設定が必要な場合は、下記のように記述する。</p>
 * <p>例、<samp>Baz</samp> クラスの <samp>void work(int, String)</samp>
 *     メソッドで、クラス内の <samp>status</samp> フィールドを参照／更新する。</p>
 * <pre><samp> HiddenAccessor acc = new HiddenAccessor(Baz.class);
 * acc.setField("status", 3);
 * acc.setMethod("work", int.class, String.class);
 * acc.invoke(10, "update");
 * assertEquals(10, acc.getField("status"));</samp></pre></li>
 * 
 * <li><p>他でテストした結果の確認にのみ使用することもできる。</p>
 * <p>例、別途 <samp>FooBar</samp> クラスのテストを行い、
 * <samp>result</samp> フィールドの値の確認だけを行う。</p>
 * <pre><samp> FooBar f = new FooBar();
 * f.run();
 * HiddenAccessor acc = new HiddenAccessor(FooBar.class);
 * acc.setInstance(f);
 * assertEquals(-1, acc.getField("result"));</samp></pre></li>
 * </ul>
 * @author Honeplus
 * @version $Id: HiddenAccessor.java 33 2009-03-01 16:29:40Z Honeplus $
 */
public class HiddenAccessor {

	/** テストするクラス */
	private Class<?> clazz;
	/** テストするクラスのオブジェクト */
	private Object object;
	/** オブジェクトを生成する際に使用するコンストラクタ */
	private Constructor<?> constructor;
	/** テストするメソッド */
	private Method method;

	/**
	 * 指定されたクラスをテストするためのオブジェクトを生成。
	 * @param type クラス
	 */
	public HiddenAccessor(final Class<?> type) {
		this.clazz = type;
	}

	/**
	 * テストするクラスのコンストラクタをを返す。
	 * @return テストするクラスのコンストラクタ
	 */
	protected Constructor<?> getConstructor() {
		// 指定されているコンストラクタを返す
		return this.constructor;
	}
	/**
	 * <p>テストするクラスのコンストラクタを指定する。</p>
	 * <p><b>コンストラクタの確保に失敗した場合は、{@link org.junit.Assert#fail()}
	 * にてテストを失敗させる。</b></p>
	 * @param parameterTypes パラメータ配列
	 */
	public void setConstructor(final Class<?>... parameterTypes) {

		try {
			// 指定されたコンストラクタを取得
			this.constructor = clazz.getDeclaredConstructor(parameterTypes);
			// 指定されたコンストラクタのアクセス権を確保
			this.constructor.setAccessible(true);
		} catch (Throwable e) {
			// メソッド指定誤り、アクセス不可も含め全てテスト失敗
			fail(e.toString());
		}
	}

	/**
	 * テストする／したクラスのオブジェクトを返す。
	 * @return テストする／したオブジェクト
	 */
	public Object getInstance() {
		// 登録されているオブジェクトを返す
		return this.object;
	}
	/**
	 * テストするオブジェクトを指定する。
	 * @param obj テストするオブジェクト
	 */
	public void setInstance(final Object obj) {
		// 指定されたオブジェクトを使用する
		this.object = obj;
	}
	/**
	 * <p>テストするクラスのオブジェクトを指定されたパラメータで生成する。</p>
	 * <p>コンストラクタには、{@link #setConstructor(Class...)} で指定されたメソッドを用いる。
	 * 指定されていない場合は、デフォルトコンストラクタを使用する。</p>
	 * <p><b>オブジェクトの生成に失敗した場合は、{@link org.junit.Assert#fail()}
	 * にてテストを失敗させる。</b></p>
	 * @param initargs コンストラクタ呼び出しに引数として渡すオブジェクトの配列。
	 *                 プリミティブ型の値は、適切な型のラッパーオブジェクト内にラップされる。
	 * @return このオブジェクトが表すコンストラクタを呼び出すことで作成される新規オブジェクト
	 * @throws Throwable コンストラクタ実行時の例外
	 */
	public Object newInstance(final Object... initargs) throws Throwable {

		try {
			// コンストラクタが指定されておらず、パラメータも0件の場合
			if (getConstructor() == null
					&& (initargs == null || initargs.length == 0)) {
				// デフォルトコンストラクタを使用する
				setConstructor();
			}
			// 指定されているコンストラクタでオブジェクトを生成
			setInstance(getConstructor().newInstance(initargs));
			// 生成したオブジェクトを返す
			return getInstance();
		} catch (InvocationTargetException e) {
			// コンストラクタが例外を投げる場合は、そのまま返す
			throw e.getCause();
		} catch (Throwable e) {
			// オブジェクトが生成できない場合、テスト失敗
			fail(e.toString());
			throw new RuntimeException(e);	// failの時点で処理は終了する、コンパイルエラー抑止
		}
	}

	/**
	 * <p>テストするメソッドを指定する。</p>
	 * <p><b>メソッドの確保に失敗した場合は、{@link org.junit.Assert#fail()}
	 * にてテストを失敗させる。</b></p>
	 * @param name メソッドの名前
	 * @param parameterTypes パラメータ配列
	 */
	public void setMethod(final String name, final Class<?>... parameterTypes) {

		try {
			// 指定されたメソッドを取得
			this.method = clazz.getDeclaredMethod(name, parameterTypes);
			// 指定されたメソッドのアクセス権を確保
			this.method.setAccessible(true);
		} catch (Throwable e) {
			// メソッド指定誤り、アクセス不可も含め全てテスト失敗
			fail(e.toString());
		}
	}

	/**
	 * <p>指定されているメソッドを実行する。</p>
	 * <p><code>static</code> メソッド以外の場合必要な呼び出し元オブジェクトには、
	 * {@link #newInstance(Object...)} または {@link #setInstance(Object)}
	 * で指定されたオブジェクトを用いる。
	 * 指定されていない場合は、可能であればデフォルトコンストラクタでオブジェクトを生成する。</p>
	 * <p><b>メソッドの実行に失敗した場合は、{@link org.junit.Assert#fail()}
	 * にてテストを失敗させる。</b></p>
	 * @param args メソッド呼び出しに使用される引数
	 * @return このオブジェクトが表すメソッドを、パラメータ
	 *         <var>args</var> を使用して実行した結果
	 * @throws Throwable メソッド実行時の例外
	 */
	public Object invoke(final Object... args) throws Throwable {

		try {
			// オブジェクトが指定されておらず、非staticメソッドの場合
			if (getInstance() == null
					&& !Modifier.isStatic(this.method.getModifiers())) {
				// デフォルトコンストラクタでオブジェクトを作成する
				// ※ コンストラクタが指定されている場合、例外
				newInstance();
			}
			// 指定されているオブジェクトでメソッドを実行
			return this.method.invoke(getInstance(), args);
		} catch (InvocationTargetException e) {
			// メソッドが例外を投げる場合は、そのまま返す
			throw e.getCause();
		} catch (Throwable e) {
			// メソッド未設定、型違いも含めて全てテスト失敗
			fail(e.toString());
			throw new RuntimeException(e);	// failの時点でテストは終了する、コンパイルエラー抑止
		}
	}

	/**
	 * <p>テストする／したクラスのオブジェクトより、指定されたフィールドの値を返す。</p>
	 * <p><b>値の取得に失敗した場合は、{@link org.junit.Assert#fail()}
	 * にてテストを失敗させる。</b></p>
	 * @param name フィールド名
	 * @return フィールドの値
	 */
	public Object getField(final String name) {
		try {
			// 指定されたフィールドを取得
			Field field = clazz.getDeclaredField(name);
			// 指定されたフィールドのアクセス権を確保
			field.setAccessible(true);
			// 指定された値をフィールドから取得する
			return field.get(getInstance());
		} catch (Throwable e) {
			// フィールド指定誤り、アクセス不可、オブジェクト無しも含め全てテスト失敗
			fail(e.toString());
			throw new RuntimeException(e);	// failの時点でテストは終了する、コンパイルエラー抑止
		}
	}
	/**
	 * <p>テストするクラスのオブジェクトで、指定されたフィールドに値を設定する。</p>
	 * <p><code>static</code> フィールド以外の場合に必要な設定先オブジェクトには、
	 * {@link #newInstance(Object...)} または {@link #setInstance(Object)}
	 * で指定されたオブジェクトを用いる。
	 * 指定されていない場合は、可能であればデフォルトコンストラクタでオブジェクトを生成する。</p>
	 * <p><b>値の設定に失敗した場合は、{@link org.junit.Assert#fail()}
	 * にてテストを失敗させる。</b></p>
	 * @param name フィールド名
	 * @param value 新しいフィールド値
	 */
	public void setField(final String name, final Object value) {
		try {
			// 指定されたフィールドを取得
			Field field = clazz.getDeclaredField(name);
			// 指定されたフィールドのアクセス権を確保
			field.setAccessible(true);
			// オブジェクトが指定されておらず、非staticフィールドの場合
			if (getInstance() == null
					&& !Modifier.isStatic(field.getModifiers())) {
				// デフォルトコンストラクタでオブジェクトを作成する
				// ※ コンストラクタが指定されている場合、例外
				newInstance();
			}
			// 指定された値をフィールドに設定する
			field.set(getInstance(), value);
		} catch (Throwable e) {
			// フィールド指定誤り、アクセス不可も含め全てテスト失敗
			fail(e.toString());
		}
	}
}
