package common;

import static org.junit.Assert.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.junit.Ignore;
import org.junit.Test;

/**
 * {@link ObjectUtility} のJUnitテストクラス。
 * @author Honeplus
 * @version $Id: ObjectUtilityTest.java 32 2009-03-01 13:59:59Z Honeplus $
 */
public class ObjectUtilityTest {

	/**
	 * コンストラクタのテスト・メソッド。
	 * @throws UnsupportedOperationException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = UnsupportedOperationException.class)
	public void testConstructor() throws Throwable {
		// privateコンストラクタを無理やりテスト
		HiddenAccessor acc = new HiddenAccessor(ObjectUtility.class);
		acc.newInstance();
	}

	/** 空のテストクラス。 */
	public static class TestClassClass { }

	/** 公開されているコンストラクタを持たないテストクラス。 */
	public static class TestClassPrivate {
		/** 非公開のコンストラクタ */
		TestClassPrivate() { }
	}

	/** テスト用のコンストラクタを持つ仮クラス。 */
	public static class TestClass {
		/** 結果確認用文字列フィールド */
		String str;
		/** 結果確認用数値フィールド */
		int num;
		/** 結果確認用浮動小数点値フィールド */
		double num2;
		/** 結果確認用オブジェクトフィールド */
		Object obj;
		/**
		 * 公開コンストラクタ（String）。
		 * @param s テスト用文字列
		 */
		public TestClass(final String s) {
			this.str = s;
		}
		/**
		 * 公開コンストラクタ（int）。
		 * @param i テスト用数値
		 */
		public TestClass(final int i) {
			this.num = i;
		}
		/**
		 * 公開コンストラクタ（double）。
		 * @param d テスト用数値
		 */
		public TestClass(final double d) {
			this.num2 = d;
		}
		/**
		 * 公開コンストラクタ（String, List）。
		 * @param s テスト用文字列
		 * @param l テスト用リスト
		 */
		public TestClass(final String s, final List<?> l) {
			this.str = "List";
			this.obj = l;
		}
		/**
		 * 公開コンストラクタ（String, ArrayList）。
		 * @param s テスト用文字列
		 * @param l テスト用リスト
		 */
		public TestClass(final String s, final ArrayList<?> l) {
			this.str = "ArrayList";
			this.obj = l;
		}
		/**
		 * 公開コンストラクタ（String, String）。
		 * @param s テスト用文字列
		 * @param s2 テスト用文字列
		 */
		public TestClass(final String s, final String s2) {
			this.str = "String";
			this.obj = s2;
		}
		/**
		 * 公開コンストラクタ（List, String...）。
		 * @param l テスト用リスト
		 * @param sVariable テスト用文字列配列
		 */
		public TestClass(final List<?> l, final String... sVariable) {
			this.str = "String...";
			this.obj = sVariable;
		}
	}

	/**
	 * {@link ObjectUtility#newInstance}
	 * のテスト・メソッド（オブジェクト生成）。
	 * @throws Throwable テスト失敗
	 */
	@Test
	public void testNewInstanceClass() throws Throwable {
		// 指定したクラスのオブジェクトが生成されること
		assertTrue(ObjectUtility.newInstance(TestClassClass.class)
				instanceof TestClassClass);
	}

	/**
	 * {@link ObjectUtility#newInstance}
	 * のテスト・メソッド（null値）。
	 * @throws IllegalArgumentException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = IllegalArgumentException.class)
	public void testNewInstanceNull() throws Throwable {
		// オブジェクトが生成できず例外となること
		ObjectUtility.newInstance(null);
	}

	/**
	 * {@link ObjectUtility#newInstance}
	 * のテスト・メソッド（非公開コンストラクタ）。
	 * @throws NoSuchMethodException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = NoSuchMethodException.class)
	public void testNewInstancePrivate() throws Throwable {
		// オブジェクトが生成できず例外となること
		ObjectUtility.newInstance(TestClassPrivate.class);
	}

	/**
	 * {@link ObjectUtility#newInstance}
	 * のテスト・メソッド（パラメータ数不一致）。
	 * @throws NoSuchMethodException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = NoSuchMethodException.class)
	public void testNewInstanceMismatch() throws Throwable {
		// オブジェクトが生成できず例外となること
		ObjectUtility.newInstance(TestClassClass.class, "test");
	}

	/**
	 * {@link ObjectUtility#newInstance}
	 * のテスト・メソッド（オブジェクト型パラメータ）。
	 * @throws Throwable テスト失敗
	 */
	@Test
	public void testNewInstanceParam() throws Throwable {
		// 指定したコンストラクタのオブジェクトが生成されること
		assertEquals("test", ((TestClass) ObjectUtility.newInstance(
				TestClass.class, "test")).str);
		assertEquals("test2", ((TestClass) ObjectUtility.newInstance(
				TestClass.class, "test", "test2")).obj);
	}

	/**
	 * {@link ObjectUtility#newInstance}
	 * のテスト・メソッド（基本型パラメータ）。
	 * @throws Throwable テスト失敗
	 */
	@Test
	public void testNewInstancePrimitive() throws Throwable {
		// 指定したコンストラクタのオブジェクトが生成されること
		assertEquals(-1, ((TestClass) ObjectUtility.newInstance(
				TestClass.class, -1)).num);
		assertEquals(Integer.valueOf(2), ((TestClass) ObjectUtility.newInstance(
				TestClass.class, Integer.valueOf(2))).num);
		assertEquals(0.1, ((TestClass) ObjectUtility.newInstance(
				TestClass.class, 0.2)).num2);
		assertEquals(2, ((TestClass) ObjectUtility.newInstance(
				TestClass.class, 2L)).num2);
	}

	/**
	 * {@link ObjectUtility#newInstance}
	 * のテスト・メソッド（実行可能なnullパラメータ）。
	 * @throws Throwable テスト失敗
	 */
	@Test
	public void testNewInstanceNullSuccess() throws Throwable {
		// 指定したコンストラクタのオブジェクトが生成されること
		assertNull(((TestClass) ObjectUtility.newInstance(
				TestClass.class, new Object[]{null})).str);
	}

	/**
	 * {@link ObjectUtility#newInstance}
	 * のテスト・メソッド（実行不可能なnullパラメータ）。
	 * @throws Throwable テスト失敗
	 */
	@Test
	public void testNewInstanceNullFail() throws Throwable {
		// 指定したコンストラクタのオブジェクトが生成されること
		ObjectUtility.newInstance(TestClass.class, "test", null);
	}

	/**
	 * {@link ObjectUtility#newInstance}
	 * のテスト・メソッド（可変長引数）。
	 * @throws Throwable テスト失敗
	 */
	@Test
	public void testNewInstanceVariable() throws Throwable {
		// 指定したコンストラクタのオブジェクトが生成されること
		// ※ さすがに可変長でさらに可変長のパラメータを判別するのは
		//    無理があったので、配列形式で渡す
		assertEquals("String...", ((TestClass) ObjectUtility.newInstance(
				TestClass.class, new ArrayList<Object>(), new String[]{})).str);
		assertEquals("String...", ((TestClass) ObjectUtility.newInstance(
				TestClass.class, new ArrayList<Object>(),
				new String[]{"test1"})).str);
		assertEquals("String...", ((TestClass) ObjectUtility.newInstance(
				TestClass.class, new ArrayList<Object>(),
				new String[]{"test1", "test2"})).str);
		assertEquals("String...", ((TestClass) ObjectUtility.newInstance(
				TestClass.class, new ArrayList<Object>(),
				new String[]{"test1", "test2", "test3"})).str);
	}

	/**
	 * {@link ObjectUtility#newInstance}
	 * のテスト・メソッド（継承クラス）。
	 * @throws Throwable テスト失敗
	 */
	@Ignore("未対応")
	@Test
	public void testNewInstanceExtends() throws Throwable {
		// 指定したコンストラクタのオブジェクトが生成されること
		assertEquals("List", ((TestClass) ObjectUtility.newInstance(
				TestClass.class, null, new LinkedList<Object>())).str);
		assertEquals("ArrayList", ((TestClass) ObjectUtility.newInstance(
				TestClass.class, null, new ArrayList<Object>())).str);
	}

	/**
	 * <p>{@link ObjectUtility#getConstructor}
	 * のテスト・メソッド（null値）。</p>
	 * <p>※ 他のテストについては、{@link ObjectUtility#newInstance}
	 * のテストメソッドでまかなわれているため、実装しない。</p>
	 * @throws NullPointerException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = NullPointerException.class)
	public void testGetConstructorNull() throws Throwable {
		// クラスが指定されていない場合、例外を投げること
		// privateメソッドを無理やりテスト
		HiddenAccessor acc = new HiddenAccessor(ObjectUtility.class);
		acc.setMethod("getConstructor", Class.class, Object[].class);
		acc.invoke(null, (Object) null);
	}

	/**
	 * {@link ObjectUtility#getProperties}
	 * のテスト・メソッド（読み込み成功）。
	 * @throws Throwable テスト失敗
	 */
	@Test
	public void testGetPropertiesSuccess() throws Throwable {
		final String testData = "message_ja.properties";
		// 存在するプロパティファイルが読み込めること
		assertNotNull(ObjectUtility.getProperties(testData));
	}

	/**
	 * {@link ObjectUtility#getProperties}
	 * のテスト・メソッド（null値）。
	 * @throws NullPointerException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = NullPointerException.class)
	public void testGetPropertiesNull() throws Throwable {
		final String testData = "nothing.properties";
		// プロパティファイルが存在しない場合、例外となること
		ObjectUtility.getProperties(testData);
	}

	/**
	 * {@link ObjectUtility#getProperties}
	 * のテスト・メソッド（ファイル無し）。
	 * @throws NullPointerException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = NullPointerException.class)
	public void testGetPropertiesNothing() throws Throwable {
		final String testData = "nothing.properties";
		// プロパティファイルが存在しない場合、例外となること
		ObjectUtility.getProperties(testData);
	}

	/**
	 * {@link ObjectUtility#getProperties}
	 * のテスト・メソッド（無効なプロパティファイル）。
	 * @throws IllegalArgumentException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Ignore("対象データ無し")
	@Test(expected = IllegalArgumentException.class)
	public void testGetPropertiesInvalid() throws Throwable {
		final String testData = "invalid.properties";
		// プロパティファイルのフォーマットが無効な場合、例外となること
		ObjectUtility.getProperties(testData);
	}

	/**
	 * {@link ObjectUtility#getProperties}
	 * のテスト・メソッド（I/Oエラー）。
	 * @throws IOException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Ignore("発生困難")
	@Test(expected = IOException.class)
	public void testGetPropertiesIOError() throws Throwable {
		fail("まだ実装されていません");
	}
}
