package common.client;

import static org.junit.Assert.*;

import java.util.IllegalFormatException;

import org.junit.Ignore;
import org.junit.Test;

import common.HiddenAccessor;
import common.client.AbstractConsole.SystemExitException;

/**
 * {@link AbstractConsole} のJUnitテストクラス。
 * @author Honeplus
 * @version $Id: AbstractConsoleTest.java 32 2009-03-01 13:59:59Z Honeplus $
 */
public class AbstractConsoleTest {

	/**
	 * {@link AbstractConsole} のテストでインスタンスを作成するための仮クラス。
	 */
	static class TestConsole extends AbstractConsole {
		/**
		 * テスト用の抽象メソッド実装。
		 * @param args 入力パラメータリスト
		 */
		@Override
		protected void parse(final String[] args) {
		}
		/**
		 * テスト用の抽象メソッド実装。
		 */
		@Override
		protected void run() {
		}
	}

	/**
	 * {@link AbstractConsole#validate} を試験用にコールするメソッド。
	 * @param args {@link AbstractConsole#validate} の <var>args</var>
	 * @throws Throwable メソッド実行時の例外
	 */
	private void runValidate(final String[] args) throws Throwable {
		// privateメソッドを無理やりテスト
		HiddenAccessor acc = new HiddenAccessor(AbstractConsole.class);
		acc.setInstance(new TestConsole());
		acc.setMethod("validate", String[].class);
		acc.invoke((Object) args);
	}

	/**
	 * {@link AbstractConsole#validate}
	 * のテスト・メソッド（null値）。
	 * @throws SystemExitException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = SystemExitException.class)
	public void testValidateNull() throws Throwable {
		// 引数にnullを渡した場合に終了例外となること
		// ※ 画面にヘルプメッセージが表示されることも試験内容だが、
		//    ここでは確認できない。
		runValidate(null);
	}

	/**
	 * {@link AbstractConsole#validate}
	 * のテスト・メソッド（空値）。
	 * @throws SystemExitException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = SystemExitException.class)
	public void testValidateEmpty() throws Throwable {
		// 引数に空の配列を渡した場合に終了例外となること
		// ※ 画面にヘルプメッセージが表示されることも試験内容だが、
		//    ここでは確認できない。
		runValidate(new String[]{});
	}

	/**
	 * {@link AbstractConsole#validate}
	 * のテスト・メソッド（パラメータ1件）。
	 * @throws Throwable テスト失敗
	 */
	@Test
	public void testValidateParam1() throws Throwable {
		// 引数に2件の配列を渡した場合に正常に処理を終えること
		runValidate(new String[]{"test"});
	}

	/**
	 * {@link AbstractConsole#parse} のテスト・メソッド。
	 */
	@Ignore("抽象メソッド")
	@Test
	public void testParse() {
		fail("まだ実装されていません");
	}

	/**
	 * {@link AbstractConsole#run()} のテスト・メソッド。
	 */
	@Ignore("抽象メソッド")
	@Test
	public void testRun() {
		fail("まだ実装されていません");
	}

	/**
	 * {@link AbstractConsole#printf} を試験用にコールするメソッド。
	 * @param format {@link AbstractConsole#printf} の <var>format</var>
	 * @param args 〃 の <var>args</var>
	 * @throws Throwable メソッド実行時の例外
	 */
	private void runPrintf(final String format, final Object... args)
		throws Throwable {
		// privateメソッドを無理やりテスト
		HiddenAccessor acc = new HiddenAccessor(AbstractConsole.class);
		acc.setInstance(new TestConsole());
		acc.setMethod("printf", String.class, Object[].class);
		acc.invoke(format, args);
	}

	/**
	 * {@link AbstractConsole#printf} のテスト・メソッド（正常）。
	 * @throws Throwable メソッド実行時の例外
	 */
	@Test
	public void testPrintfSuccess() throws Throwable {
		// ※ 画面にメッセージが表示されることも試験内容だが、ここでは確認できない。
		//    エラーとならないことだけ確認。
		// ↑ コンソールからしか動かないため、JUnitでは最低限の試験しかできない。
		runPrintf("test messaage");
		runPrintf("テストメッセージ");
		runPrintf("test %s.", "inline");
		runPrintf("int %d.", 1);
	}

	/**
	 * {@link AbstractConsole#printf}
	 * のテスト・メソッド（不正な引数）。
	 * @throws Throwable メソッド実行時の例外
	 */
	@Ignore("コンソールからしか動かないため、JUnitでは試験できない")
	@Test(expected = IllegalFormatException.class)
	public void testPrintfIllegalFormat() throws Throwable {
		runPrintf("int %d.", "inline");
	}

	/**
	 * {@link AbstractConsole#abstractMain} を試験用にコールするメソッド。
	 * @param clazz {@link AbstractConsole#abstractMain} の <var>clazz</var>
	 * @param args 〃 の <var>args</var>
	 * @throws Throwable メソッド実行時の例外
	 */
	private void runAbstractMain(
			final Class<? extends AbstractConsole> clazz,
			final String[] args) throws Throwable {
		// privateメソッドを無理やりテスト
		HiddenAccessor acc = new HiddenAccessor(AbstractConsole.class);
		acc.setInstance(new TestConsole());
		acc.setMethod("abstractMain", Class.class, String[].class);
		acc.invoke(clazz, args);
	}

	/**
	 * {@link AbstractConsole#abstractMain のテスト・メソッド。
	 * @throws Throwable メソッド実行時の例外
	 */
	@Ignore("System.exit() により停止してしまうため、JUnitでは試験できない")
	@Test
	public void testAbstractMain() throws Throwable {
		runAbstractMain(TestConsole.class, new String[]{"test"});
	}
}
