package converter.client;

import static org.junit.Assert.*;

import org.junit.Test;
import org.junit.Ignore;

import common.HiddenAccessor;
import common.client.AbstractConsole.SystemExitException;
import converter.text.TextConvertUtility;

/**
 * {@link TextConvertConsole} のJUnitテストクラス。
 * @author Honeplus
 * @version $Id: TextConvertConsoleTest.java 32 2009-03-01 13:59:59Z Honeplus $
 */
public class TextConvertConsoleTest {

	/**
	 * コンストラクタのテスト・メソッド。
	 */
	@Test
	public void testConstructor() {
		// 特に何も無し。例外とならないこと
		new TextConvertConsole();
	}

	/**
	 * {@link TextConvertConsole#validate} を試験用にコールするメソッド。
	 * @param args {@link TextConvertConsole#validate} の <var>args</var>
	 * @throws Throwable メソッド実行時の例外
	 */
	private void runValidate(final String[] args) throws Throwable {
		// privateメソッドを無理やりテスト
		HiddenAccessor acc = new HiddenAccessor(TextConvertConsole.class);
		acc.setMethod("validate", String[].class);
		acc.invoke((Object) args);
	}

	/**
	 * {@link TextConvertConsole#validate}
	 * のテスト・メソッド（null値）。
	 * @throws SystemExitException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = SystemExitException.class)
	public void testValidateNull() throws Throwable {
		// 引数にnullを渡した場合に終了例外となること
		// ※ 画面にヘルプメッセージが表示されることも試験内容だが、
		//    ここでは確認できない。
		runValidate(null);
	}

	/**
	 * {@link TextConvertConsole#validate}
	 * のテスト・メソッド（空値）。
	 * @throws SystemExitException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = SystemExitException.class)
	public void testValidateEmpty() throws Throwable {
		// 引数に空の配列を渡した場合に終了例外となること
		// ※ 画面にヘルプメッセージが表示されることも試験内容だが、
		//    ここでは確認できない。
		runValidate(new String[]{});
	}

	/**
	 * {@link TextConvertConsole#validate}
	 * のテスト・メソッド（パラメータ1件）。
	 * @throws SystemExitException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = SystemExitException.class)
	public void testValidateArgs1() throws Throwable {
		// 引数に1件の配列を渡した場合に終了例外となること
		// ※ 標準エラー出力にエラーメッセージが表示されることも試験内容だが、
		//    ここでは確認できない。コンソールのログを見て確認すること。
		runValidate(new String[]{"test"});
	}

	/**
	 * {@link TextConvertConsole#validate}
	 * のテスト・メソッド（パラメータ2件）。
	 * @throws Throwable テスト失敗
	 */
	@Test
	public void testValidateArgs2() throws Throwable {
		// 引数に2件の配列を渡した場合に正常に処理を終えること
		runValidate(new String[]{"test", "test"});
	}

	/**
	 * {@link TextConvertConsole#parse} を試験用にコールするメソッド。
	 * @param expectedText {@link TextConvertConsole#parse} で設定される変換元文字列期待値
	 * @param expectedRunType 〃 で設定される変換処理期待値
	 * @param args {@link TextConvertConsole#parse} の <var>args</var>
	 * @throws Throwable メソッド実行時の例外
	 */
	private void runParse(final String expectedText,
			final TextConvertUtility.RunType expectedRunType,
			final String[] args) throws Throwable {
		// privateメソッドを無理やりテスト
		HiddenAccessor acc = new HiddenAccessor(TextConvertConsole.class);
		acc.setMethod("parse", String[].class);
		acc.invoke((Object) args);
		assertEquals(expectedText, acc.getField("text"));
		assertEquals(expectedRunType, acc.getField("runType"));
	}

	/**
	 * {@link TextConvertConsole#parse}
	 * のテスト・メソッド（正常パターン）。
	 * @throws Throwable テスト失敗
	 */
	@Test
	public void testParseSuccess() throws Throwable {
		runParse("012ABCあいう",
				TextConvertUtility.RunType.HALFWIDTH_TO_FULLWIDTH,
				new String[]{"-f", "012ABCあいう"});
		runParse("012 ABC あいう",
				TextConvertUtility.RunType.HALFWIDTH_TO_FULLWIDTH,
				new String[]{"-F", "012", "ABC", "あいう"});
		runParse("０１２ＡＢＣ",
				TextConvertUtility.RunType.FULLWIDTH_TO_HALFWIDTH,
				new String[]{"/h", "０１２ＡＢＣ"});
		runParse("０１２ＡＢＣ",
				TextConvertUtility.RunType.FULLWIDTH_TO_HALFWIDTH,
				new String[]{"-H", "０１２ＡＢＣ"});
	}

	/**
	 * {@link TextConvertConsole#parse}
	 * のテスト・メソッド（オプション不正）。
	 * @throws SystemExitException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = SystemExitException.class)
	public void testParseOptionFail() throws Throwable {
		// ※ 標準エラー出力にエラーメッセージが表示されることも試験内容だが、
		//    ここでは確認できない。コンソールのログを見て確認すること。
		runParse("012ABCあいう",
				TextConvertUtility.RunType.HALFWIDTH_TO_FULLWIDTH,
				new String[]{"-G", "012ABCあいう"});
	}

	/**
	 * {@link TextConvertConsole#parse}
	 * のテスト・メソッド（オプション無し）。
	 * @throws SystemExitException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = SystemExitException.class)
	public void testParseOptionEmpty() throws Throwable {
		// ※ 標準エラー出力にエラーメッセージが表示されることも試験内容だが、
		//    ここでは確認できない。コンソールのログを見て確認すること。
		runParse("012ABCあいう",
				TextConvertUtility.RunType.HALFWIDTH_TO_FULLWIDTH,
				new String[]{"012ABCあいう", "０１２ＡＢＣ"});
	}

	/**
	 * {@link TextConvertConsole#run} を試験用にコールするメソッド。
	 * @param text {@link TextConvertConsole#run} で使用する変換元文字列
	 * @param runType 〃 で使用する変換処理
	 * @throws Throwable メソッド実行時の例外
	 */
	private void runRun(final String text,
			final TextConvertUtility.RunType runType) throws Throwable {
		// privateメソッドを無理やりテスト
		HiddenAccessor acc = new HiddenAccessor(TextConvertConsole.class);
		acc.setMethod("run");
		acc.setField("text", text);
		acc.setField("runType", runType);
		acc.invoke();
	}

	/**
	 * {@link TextConvertConsole#run}
	 * のテスト・メソッド。
	 * @throws Throwable テスト失敗
	 */
	@Test
	public void testRun() throws Throwable {
		// ※ 標準出力に結果が表示されることも試験内容だが、
		//    ここでは確認できない。コンソールのログを見て確認すること。
		runRun("012ABCあいう",
				TextConvertUtility.RunType.HALFWIDTH_TO_FULLWIDTH);
		runRun("ⅢⅣⅴⅵ①②",
				TextConvertUtility.RunType.ROMAN_AND_ENCIRCLED_HALFWIDTH);
	}

	/**
	 * {@link TextConvertConsole#main}
	 * のテスト・メソッド。
	 */
	@Ignore("System.exit() により停止してしまうため、JUnitでは試験できない")
	@Test
	public void testMain() {
		TextConvertConsole.main(null);
	}
}
