package converter.text;

import static org.junit.Assert.*;

import java.util.Date;

import org.junit.Test;

import common.HiddenAccessor;

/**
 * {@link TextConvertUtility} のJUnitテストクラス
 * （パラメータでの正常系のパターンテスト）。
 * @author Honeplus
 * @version $Id: TextConvertUtilityTest.java 32 2009-03-01 13:59:59Z Honeplus $
 */
public class TextConvertUtilityTest {

	/**
	 * コンストラクタのテスト・メソッド。
	 * @throws UnsupportedOperationException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = UnsupportedOperationException.class)
	public void testConstructor() throws Throwable {
		// privateコンストラクタを無理やりテスト
		HiddenAccessor acc = new HiddenAccessor(TextConvertUtility.class);
		acc.newInstance();
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * を速度測定用に繰り返しコールするメソッド。
	 * @param text {@link TextConvertUtility#convert} の <var>text</var>
	 * @param runType 〃 の <var>runType</var>
	 * @param expected 期待値
	 * @param loop 繰り返し実行回数
	 */
	private void runConvert(final String text,
			final TextConvertUtility.RunType runType,
			final String expected, final int loop) {
		// 適当な回数ループして速度を測定。
		// 何秒まで許容するかは呼び出し元でチェック
		Date start = new Date();
		for (int i = 0; i < loop; i++) {
			assertEquals(expected,
					TextConvertUtility.convert(text, runType));
		}
		System.out.println(runType.name() + " : "
				+ (new Date().getTime() - start.getTime()) + " ms");
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#FULLWIDTH_NUMBER_TO_HALFWIDTH}
	 * ためのテスト・メソッド（全角数字→半角数字）。
	 */
	@Test(timeout = 1000)
	public void testConvertFullwidthNumber2Halfidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility.RunType.FULLWIDTH_NUMBER_TO_HALFWIDTH,
				"012abcABC.+-012ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#FULLWIDTH_LATIN_TO_HALFWIDTH}
	 * ためのテスト・メソッド（全角英字→半角英字）。
	 */
	@Test(timeout = 1000)
	public void testConvertFullwidthLatin2Halfidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility.RunType.FULLWIDTH_LATIN_TO_HALFWIDTH,
				"012abcABC.+-０１２abcABC．＋－あいうｱｲｳアイウ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#FULLWIDTH_LATIN_AND_NUMBER_TO_HALFWIDTH}
	 * ためのテスト・メソッド（全角英数→半角英数）。
	 */
	@Test(timeout = 1000)
	public void testConvertFullwidthLatinAndNumber2Halfidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility
					.RunType.FULLWIDTH_LATIN_AND_NUMBER_TO_HALFWIDTH,
				"012abcABC.+-012abcABC．＋－あいうｱｲｳアイウ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#FULLWIDTH_MARK_TO_HALFWIDTH}
	 * ためのテスト・メソッド（全角記号→半角記号）。
	 */
	@Test(timeout = 10000)
	public void testConvertFullwidthMark2Halfidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility.RunType.FULLWIDTH_MARK_TO_HALFWIDTH,
				"012abcABC.+-０１２ａｂｃＡＢＣ.+-あいうｱｲｳアイウ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#FULLWIDTH_LATIN_AND_NUMBER_AND_MARK_TO_HALFWIDTH}
	 * ためのテスト・メソッド（全角英数記号→半角英数記号）。
	 */
	@Test(timeout = 1000)
	public void testConvertFullwidthLatinAndNumberAndMark2Halfidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility
					.RunType.FULLWIDTH_LATIN_AND_NUMBER_AND_MARK_TO_HALFWIDTH,
				"012abcABC.+-012abcABC.+-あいうｱｲｳアイウ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#FULLWIDTH_KANA_TO_HALFWIDTH}
	 * ためのテスト・メソッド（全角カナ→半角カナ）。
	 */
	@Test(timeout = 1000)
	public void testConvertFullwidthKana2Halfidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility.RunType.FULLWIDTH_KANA_TO_HALFWIDTH,
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳｱｲｳ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#FULLWIDTH_TO_HALFWIDTH}
	 * ためのテスト・メソッド（全角→半角（可能なもの全て））。
	 */
	@Test(timeout = 1000)
	public void testConvertFullwidth2Halfidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility.RunType.FULLWIDTH_TO_HALFWIDTH,
				"012abcABC.+-012abcABC.+-あいうｱｲｳｱｲｳ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#HALFWIDTH_NUMBER_TO_FULLWIDTH}
	 * ためのテスト・メソッド（半角数値→全角数値）。
	 */
	@Test(timeout = 1000)
	public void testConvertHalfidthNumber2Fullwidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility.RunType.HALFWIDTH_NUMBER_TO_FULLWIDTH,
				"０１２abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#HALFWIDTH_LATIN_TO_FULLWIDTH}
	 * ためのテスト・メソッド（半角英字→全角英字）。
	 */
	@Test(timeout = 1000)
	public void testConvertHalfidthLatin2Fullwidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility.RunType.HALFWIDTH_LATIN_TO_FULLWIDTH,
				"012ａｂｃＡＢＣ.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#HALFWIDTH_LATIN_AND_NUMBER_TO_FULLWIDTH}
	 * ためのテスト・メソッド（半角英数→全角英数）。
	 */
	@Test(timeout = 1000)
	public void testConvertHalfidthLatinAndNumber2Fullwidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility
					.RunType.HALFWIDTH_LATIN_AND_NUMBER_TO_FULLWIDTH,
				"０１２ａｂｃＡＢＣ.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#HALFWIDTH_MARK_TO_FULLWIDTH}
	 * ためのテスト・メソッド（半角記号→全角記号）。
	 */
	@Test(timeout = 1000)
	public void testConvertHalfidthMark2Fullwidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility.RunType.HALFWIDTH_MARK_TO_FULLWIDTH,
				"012abcABC．＋－０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#HALFWIDTH_LATIN_AND_NUMBER_AND_MARK_TO_FULLWIDTH}
	 * ためのテスト・メソッド（半角英数記号→全角英数記号）。
	 */
	@Test(timeout = 1000)
	public void testConvertHalfidthLatinAndNumberAndMark2Fullwidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility
					.RunType.HALFWIDTH_LATIN_AND_NUMBER_AND_MARK_TO_FULLWIDTH,
				"０１２ａｂｃＡＢＣ．＋－０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#HALFWIDTH_KANA_TO_FULLWIDTH}
	 * ためのテスト・メソッド（半角カナ→全角カナ）。
	 */
	@Test(timeout = 1000)
	public void testConvertHalfidthKana2Fullwidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility.RunType.HALFWIDTH_KANA_TO_FULLWIDTH,
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうアイウアイウ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#HALFWIDTH_TO_FULLWIDTH}
	 * ためのテスト・メソッド（半角→全角（可能なもの全て））。
	 */
	@Test(timeout = 1000)
	public void testConvertHalfidth2Fullwidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility.RunType.HALFWIDTH_TO_FULLWIDTH,
				"０１２ａｂｃＡＢＣ．＋－０１２ａｂｃＡＢＣ．＋－あいうアイウアイウ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#HIRAGANA_TO_KATAKANA}
	 * ためのテスト・メソッド（ひらがな→カタカナ）。
	 */
	@Test(timeout = 1000)
	public void testConvertHiragana2Katakana() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility.RunType.HIRAGANA_TO_KATAKANA,
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－アイウｱｲｳアイウ亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#KATAKANA_TO_HIRAGANA}
	 * ためのテスト・メソッド（カタカナ→ひらがな）。
	 */
	@Test(timeout = 1000)
	public void testConvertKatakana2testConvertHiragana2Katakana() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇",
				TextConvertUtility.RunType.KATAKANA_TO_HIRAGANA,
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳあいう亜伊宇",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#ROMAN_NUMERALS_TO_HALFWIDTH}
	 * ためのテスト・メソッド（ローマ数字→似た半角英字文字列）。
	 */
	@Test(timeout = 1000)
	public void testConvertRomanNumerals2Halfwidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇Ⅰ①⑴",
				TextConvertUtility.RunType.ROMAN_NUMERALS_TO_HALFWIDTH,
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇I①⑴",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#ENCIRCLED_NUMBER_HALFWIDTH}
	 * ためのテスト・メソッド（半角→全角（可能なもの全て））。
	 */
	@Test(timeout = 1000)
	public void testConvertEncircled2Halfwidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇Ⅰ①⑴",
				TextConvertUtility.RunType.ENCIRCLED_NUMBER_HALFWIDTH,
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇Ⅰ(1)⑴",
				10000);
	}

	/**
	 * {@link TextConvertUtility#convert(String, TextConvertUtility.RunType)}
	 * の {@link TextConvertUtility.RunType#ROMAN_AND_ENCIRCLED_HALFWIDTH}
	 * ためのテスト・メソッド（ローマ数字・丸数字→似た半角英数文字列）。
	 */
	@Test(timeout = 1000)
	public void testConvertRomanAndEncircled2Halfwidth() {
		// 10000回1秒以内なら許容
		runConvert(
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇Ⅰ①⑴",
				TextConvertUtility.RunType.ROMAN_AND_ENCIRCLED_HALFWIDTH,
				"012abcABC.+-０１２ａｂｃＡＢＣ．＋－あいうｱｲｳアイウ亜伊宇I(1)⑴",
				10000);
	}


	/**
	 * {@link TextConvertUtility#getInstance}
	 * を試験用にコールするメソッド。
	 * @param type {@link TextConvertUtility#getInstance} の <var>type</var>
	 * @param param 〃 の <var>param</var>
	 * @return 〃 の 戻り値
	 * @throws Throwable メソッド実行時の例外
	 */
	private TextConverter runGetInstance(
			final Class<? extends TextConverter> type, final Object... param)
			throws Throwable {
		// privateメソッドを無理やりテスト
		HiddenAccessor acc = new HiddenAccessor(TextConvertUtility.class);
		acc.setMethod("getInstance", Class.class, Object[].class);
		return (TextConverter) acc.invoke(type, param);
	}

	/**
	 * {@link TextConvertUtility#getInstance}
	 * のテスト・メソッド（null値）。
	 * @throws RuntimeException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = RuntimeException.class)
	public void testGetInstanceNull() throws Throwable {
		// クラスにnullを渡した場合に例外となること
		runGetInstance(null);
	}

	/**
	 * {@link TextConvertUtility#getInstance}
	 * のテスト・メソッド（存在しないコンストラクタ）。
	 * @throws RuntimeException テスト成功
	 * @throws Throwable テスト失敗
	 */
	@Test(expected = RuntimeException.class)
	public void testGetInstanceNoSuchMethod() throws Throwable {
		// 存在しないコンストラクタを指定した場合に、例外となること
		runGetInstance(FullwidthLatin2Halfwidth.class, "dummy");
	}

	/**
	 * {@link TextConvertUtility#getInstance}
	 * のテスト・メソッド（オブジェクト）。
	 * @throws Throwable テスト失敗
	 */
	@Test
	public void testGetInstanceObject() throws Throwable {
		final String testData
			= "converter/text/halfwidthMark2fullwidth.properties";
		// 期待するオブジェクトが生成されること
		assertTrue(runGetInstance(HalfwidthKana2Fullwidth.class)
				instanceof HalfwidthKana2Fullwidth);
		assertTrue(runGetInstance(PropertiesMappingConverter.class, testData)
				instanceof PropertiesMappingConverter);
		assertTrue(runGetInstance(FullwidthLatin2Halfwidth.class)
				instanceof FullwidthLatin2Halfwidth);
	}
	/**
	 * {@link TextConvertUtility#getInstance}
	 * のテスト・メソッド（キャッシュ）。
	 * @throws Throwable テスト失敗
	 */
	@Test
	public void testGetInstanceCache() throws Throwable {
		final String testData
			= "converter/text/halfwidthMark2fullwidth.properties";
		// キャッシュ対象のクラスが、2度呼んでオブジェクトがキャッシュされていること
		assertSame(runGetInstance(HalfwidthKana2Fullwidth.class),
				runGetInstance(HalfwidthKana2Fullwidth.class));
		assertSame(runGetInstance(PropertiesMappingConverter.class, testData),
				runGetInstance(PropertiesMappingConverter.class, testData));
		// キャッシュ対象のクラスでも、違うクラスなら違う結果となること
		assertNotSame(runGetInstance(HalfwidthKana2Fullwidth.class),
				runGetInstance(PropertiesMappingConverter.class, testData));
		// キャッシュ対象外のクラスが、2度呼んでオブジェクトがキャッシュされていないこと
		assertNotSame(runGetInstance(FullwidthLatin2Halfwidth.class),
				runGetInstance(FullwidthLatin2Halfwidth.class));
	}
}
